
c                       Computer Program EIGEN1
c                           January 2016 

c			     by A. Ghali
c               Emeritus Professor of Civil Engineering
c                      The University of Calgary
c                       Calgary, Canada T2N 1N4

c   Companion of A. Ghali and Neville A. M., Structural Analysis: A
c  Unified Classical and Matrix Approach,CRC Press, Spon Press,7th ed.,2016.

c  Calculation of the eigenvalues and the eigenvectors of stiffness matrix,
c  [S] and diagonal mass matrix [m],based on Ghali,A., Neville,A. M., Section
c  A3.1. Reference: Chopra, A. K., Dynamics of Structures,Theory and
c  Applications to Earthquake Engineering, 4th ed., Printice Hall 2012.

c      The problem is finding K-eigenvalues, Lamda and the corresponding
c      eigenvectors satisfying the equations:

c      [S] {Phi}= Lamda [m] {Phi}

c      where matrices [S] and [m] are square of order N.

c      Input data instructions:

c      Prepare the input in file EIGEN1.IN, A title, less than 70 characters
c      occupies the first line. Give N and K in one line. Specify a tolerence
c      value and a limit on the number of iteration on one line. Give rows of [S], starting a new  line for
c      each row. Give rows of [m], starting a new  line for each row. On
c      a new line, give K-guess eigenvalues. K can be greater than N, but
c      the maximum number of N- non repeted eigenvalue and eigenvector pairs
c      that can be found does not exceed N. If a guess-eigenvalue is exactly
c      equal to an actual eigenvalue, the computer will stop indicating a
c      error (because the solution procedure will attempt to solve
c      simultaneous equations involving a singular matrix. Re-run the program
c      changing the guess-eigenvalue slightly. The tolerence= the ratio of the
c      change in eigenvalue dtermined in the last iteration to the new value.

c      To run the program, type 'EIGEN1' and press enter. The result will
c      be in a file named EIGEN.OUT.

c     Example of an input file:


c     Example 
c     3  3  Size of matrices [S] and [m]; number of guess-eigenvalues
c     1.E-3  5                      Tolerence, limit on number of iterations

c   298.6667    -130.6667   0.      Rows of [S] Stiffness matrix
c  -130.6667    186.6667  -56.0
c     0.        -56.       56.  

c     0.259       0.        0.      Rows of [m], mass matrix
c     0.          0.259     0.
c     0.          0.        0.1295

c     100.   600.  1500.            Guess eigenvalues 

c   Notes: The guess-eigenvaluesin the above example are chosen by Chopra.
c   In absence of such guidance, the first trial-value can be zero,
c   followed by K-trial values, with K grater than N; repeat as necassary
c   to obtain non-repetetive pairs of eigenvalue and corresponding
c   eigenvector. Program EIGEN2 does not rquires no guessed values. It can 
c   substitute EIGEN1 when [S] and [m] have no zero element on the diagonal.
c________________________________________________________________________
c                Listing of program EIGEN1

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXQ=20000)
      COMMON/REALV/Q(MAXQ)
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='EIGEN1.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='EIGEN1.OUT')
      DO 70 I=1,MAXQ
70    Q(I)=0.0

      READ(5,101)TITLE
101   FORMAT(A76)
      WRITE(6,201)TITLE
201   FORMAT(1X,A76)

      READ(5,*)N, K
      WRITE(6,100)N, K
100   FORMAT(/,1X,
     +'Calculation of the eigenvalues and eigenvectors, solution of:',/,
     +1X,' [S] {Phi}= Lamda [m] {Phi}',/,
     +1X,'Number of rows or columns in each of [S] and [m]=',1X,I5,/,
     +1X,'Number of eigenvalues or eigenvectors required=',1X,I5)
      READ(5,*)TOLER, MAXITR
      WRITE(6,200)TOLER, MAXITR
200   FORMAT(/,1x,'Change in eigenvalue in last iteration divided by',
     +/,1X,'last value= ',E11.5,
     +/,1X,'Limit on number of iterations= ',I2)


c     Elements of {Q} at which real arrays start are:
c     [S]:N1  ;  [Am]:N2  ; {XJ}:N3 ; {XJ1}:N4 ; [Phi]:N5 ; {Alamda}:N6
c     {SHIFT}:N7

      M=1

      N1=1
      N2= N*N+ N1
      N3= N*N+ N2
      N4= N+N3
      N5= N+N4
      N6= N*K+N5
      N7= N6+N

c     CALL DATA(N,S, Am, SHIFT,K)
      CALL DATA(N,Q(N1),Q(N2),Q(N7),K)

11    DO 50 IK=1,K
      WRITE(6,111)IK,Q(N7+IK-1)
111   FORMAT(/,'Search for eigenvalue and eigenvector no.',I2,1X,
     +'; trial eigenvalue=',E11.5,/)


12    CONTINUE

      DO 10 I=1,N
10    Q(N3+I-1)=1.



      ITER=0
      Amdaj=Q(N7+IK-1)


20    CONTINUE
      ITER=ITER+1

c     Determine {XJ1} by solving: ([S]-Amdaj[Am]){Xj1}=[Am]{XJ}
c     CALL Step1(S,Am,Amdaj,XJ,XJ1,N,M)
      CALL Step1(Q(N1),Q(N2),Amdaj,Q(N3),Q(N4),N,M)

c     Obtain an estimate ofthe eigenvalue and the shift for the next
c     iteration.
c     CALL Step2(XJ1,Am,XJ,Amdaj1,N,Amdaj)
      CALL Step2(Q(N4),Q(N2),Q(N3),Amdaj1,N,Amdaj)


c     Normalize {XJ1}
c     CALL Normalize(Am,XJ1,N)
      CALL Normalize(Q(N2),Q(N4),N)

      WRITE(6,120)Amdaj1,(Q(N4+II-1),II=1,N)
120   FORMAT(/,'Eigenvalue= ',E11.5,/,'Eigenvector: {',5(E11.5,1X),'}',
     +/)



      IF(ABS((Amdaj1-Amdaj)/Amdaj1).GT.TOLER)THEN
      Amdaj=Amdaj1
      DO 30 I=1,N
30    Q(N3+I-1)= Q(N4+I-1)
      IF(ITER.LE.MAXITR)GO TO 20
      END IF


      Q(N6+IK-1)=Amdaj1
      DO 40 I=1,N
40    Q(N5+(IK-1)*N+I-1)=Q(N4+I-1)

      WRITE(6,501)
501   FORMAT(/,'_____________________________________________________')
50    CONTINUE


      CALL RESULTS(Q(N6),Q(N5),N,K)


      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATA(N,S,Am,SHIFT,K)
c     Reading input matrices [S], [Am] 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(N,N),Am(N,N),SHIFT(N)
      WRITE(6,100)
100   FORMAT(/,1X,'The given matrices:',/)

      WRITE(6,300)
      DO 10 I=1,N
      READ(5,*)(S(I,J),J=1,N)
10    WRITE(6,500)(S(I,J),J=1,N)
300   FORMAT(1X,'Matrix [S] (e. g. stiffness matrix)',/)

500   FORMAT(1X,6(E11.5,1X))

      WRITE(6,200)
      DO 20 I=1,N
      READ(5,*)(Am(I,J),J=1,N)
20    WRITE(6,600)(Am(I,J),J=1,n)
200   FORMAT(/,1X,'Matrix [m] (e. g. mass matrix)',/)
600   FORMAT(1X,6(E11.5,1X))

      READ(5,*)(SHIFT(I),I=1,K)
      WRITE(6,700)(SHIFT(I),I=1,K)
700   FORMAT(/,1X,'Origin shifts (guessed) eigenvalues:',
     + /,/,6(E11.5,1X),/)

      WRITE(6,800)
800   FORMAT(/,
     +' _______________________________________________________________
     +',/,/,'Iterative calculations:',/)

      RETURN
      END


      SUBROUTINE RESULTS(Alamda,Phi,N,k)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Alamda(k),Phi(N,k)
      WRITE(6,400)
400   FORMAT(/,1x,'Summary of results:',/)
      WRITE(6,100)(Alamda(I),I=1,k)
100   FORMAT(/,1X,'Eigenvalues:',/,/,6(1X,E11.5))
      WRITE(6,300)     
300   FORMAT(/,1X,'Eigenvectors:',/)
      DO 10 I=1,N
      WRITE(6,200)(Phi(I,J),J=1,k)
200   FORMAT(6(1X,E11.5))
10    WRITE(6,201)
201   FORMAT(/)
      WRITE(6,202)N
202   FORMAT(1X,'Note: Repeated eigenvalues and crresponding eigenvector
     +s should be ',/,1X,' disregarded.The maximum number of non-repeate
     +d eigenvalues = ',I2)


      RETURN
      END

      SUBROUTINE Step2(XJ1,Am,XJ,Amdaj1,N,Amdaj)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION XJ1(N),Am(N,N),XJ(N),P(1,N)

      Sum1=0.
      Sum2=0.
      DO 5 J=1,N
5     P(1,J)=0.

      DO 10 J=1,N
      DO 10 I=1,N 
10    P(1,J)=P(1,J)+XJ1(I)*Am(I,J)

      DO 20 I=1,N
20    Sum1=Sum1+P(1,I)*XJ(I)


      DO 30 I=1,N
30    Sum2=Sum2+P(1,I)*XJ1(I)

      Amdaj1=Amdaj+Sum1/Sum2
      
      RETURN
      END

      SUBROUTINE Step1(S,Am,Amdaj,XJ,XJ1,N,M)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(N,N),Am(N,N),XJ(N),A(N,N),B(N,M),XJ1(N)

      DO 110 I=1,N
      DO 110 J=1,N
110   A(I,J)=S(I,J)

      DO 111 I=1,N
      DO 111 J=1,N
111   A(I,J)=A(I,J)-Amdaj*Am(I,J)

      DO 112 I=1,N
      SUM=0.
      DO 116 J=1,N
116   SUM=SUM+Am(I,J)*XJ(J)
112   B(I,1)=SUM

c      write(6,115)(Xj(J),J=1,N)
115   format(/,'{XJ}:',5(E11.5,1X))


      do 114 i=1,N
114   write(6,113)(A(I,J),J=1,N),B(I,1) 
113   format('Eqns to slv.:',6(e10.4,1x))
  
      DO 10 J=2,N
10    A(1,J)=A(1,J)/A(1,1)
      DO 20 J=1,M
20    B(1,J)=B(1,J)/A(1,1)
c     Remaining rows of auxiliary matrix
      DO 30 I=2,N
      DO 30 J=2,N
      IF(J.GT.I)GO TO 36
      SUM=0
      DO 35 K=1,J-1
35    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=A(I,J)-SUM
      GO TO 38
36    SUM=0
      DO 37 K=1,I-1
37    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=(A(I,J)-SUM)/A(I,I)
38    CONTINUE
30    CONTINUE
      DO 40 I=2,N
      DO 40 J=1,M
      SUM=0
      DO 45 K=1,I-1
45    SUM=SUM+A(I,K)*B(K,J)
      B(I,J)=(B(I,J)-SUM)/A(I,I)
40    CONTINUE
c     Back substitution; store [X] in [B].
      DO 50 L=1,M
      B(N,L)=B(N,L)
      DO 50 I=2,N
      IB=N-I+1
      SUM=0
      DO 55 K=IB+1,N
55    SUM=SUM+A(IB,K)*B(K,L)
50    B(IB,L)=B(IB,L)-SUM
      WRITE(6,400)
400   FORMAT(/,3X,'Vector {XJ1}:' )
      DO 60 I=1,N
60    WRITE(6,500)I,(B(I,J),J=1,M)
500   FORMAT(3X,I4,2X,5(E11.5,1X))


      DO 61 I=1,N
61    XJ1(I)=B(I,1)
      RETURN
      END

      SUBROUTINE Normalize(Am,XJ1,N)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Am(N,N),XJ1(N),P(1,N)

      Sum2=0.
      DO 5 J=1,N
5     P(1,J)=0.

      DO 10 J=1,N
      DO 10 I=1,N 
10    P(1,J)=P(1,J)+XJ1(I)*Am(I,J)

  
      DO 30 I=1,N
30    Sum2=Sum2+P(1,I)*XJ1(I)

      DO 40 I=1,N
40    XJ1(I)=XJ1(I)/SQRT(SUM2)

      RETURN
      END


