
c                       Computer Program EIGEN2
c                           January 2016 

c			     by A. Ghali
c               Emeritus Professor of Civil Engineering
c                      The University of Calgary
c                       Calgary, Canada T2N 1N4

c   Companion of A. Ghali and Neville A. M., Structural Analysis: A
c  Unified Classical and Matrix Approach,CRC Press, Spon Press,7th ed.,2016.

c  Calculation of the eigenvalues and the eigenvectors of stiffness matrix,
c  [S] and  mass matrix [m],based on Bathe, K-J, Finite Element
c  Procedures, Printice Hall, 1996,Sec. 11.3

c      The problem is finding N-eigenvalues,  and the corresponding
c      eigenvectors satisfying the equations:

c      [S] {Phi}= Lamda [m] {Phi}

c      where matrices [S] and [m] are symmetrical of order N. [S] is positive
c      definite. No zero value on the diagonal of [S] or [m].

c      Input data instructions:

c      Prepare the input in file EIGEN2.IN, A title, less than 70 characters
c      occupies the first line. Give N in one line.  On one line, specify a
c      tolerence value, RTOL (usually set to 10.**-12) and NSmax ( allowed
c      number of sweeps (usually set to 15) and the integer 1 or 0; the
c      integer 1 for printing during iterations. Giver  rows of [S] (starting
c      a new line for each row. Give rows of [m] (starting a new  line for
c      each row.


c      To run the program, type 'EIGEN2' and press enter. The result will
c      be in a file named EIGEN2.OUT. The output consists of:


c      Summary of results:
c      EIGV(N)= eigenvalues
c      X(N,N)= Eigenvectors

c     Example of an input file:


c     Example 11.9 of Bathe 
c     4    Number of rows or cols. in [S] or [m]
c     10.E-12 15  1  Tolerence, max. number of iterations, print during iters.

c     5.    -4.   1.   0.    Rows of [S] Stiffness matrix
c    -4.     6.  -4.   1.
c     1.    -4.   6.  -4.
c     0.     1.  -4.   5. 

c     1.     0.   0.   0.    Rows of [m], mass matrix
c     0.     1.   0.   0.
c     0.     0.   1.   0.
c     0.     0.   0.   1.


c________________________________________________________________________
c     Lsting of EIGEN2.FOR

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXQ=20000, MAXNI=30)
      COMMON/REALV/Q(MAXQ)
      COMMON/NINTGR/NI(MAXNI)
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='EIGEN2.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='EIGEN2.OUT')

      
      DO 70 I=1,MAXQ
70    Q(I)=0.0
      DO 71 I=1,MAXNI
71    NI(I)=0



      READ(5,101)TITLE
101   FORMAT(A76)
      WRITE(6,201)TITLE
201   FORMAT(1X,A76)

      READ(5,*)N
      WRITE(6,100)N,N
100   FORMAT(1X,
     +'Calculation of the eigenvalues and eigenvectors solution of:',/,
     +1X,' [S] {Phi}= Lamda [m] {Phi}',/,
     +1X,'Number of rows or columns in each of [S] and [m]=',1X,I5,/,
     +1X,'Number of eigenvalues or eigenvectors required=',1X,I5)
      READ(5,*)RTOL, NSmax,IFPR
      WRITE(6,200)RTOL,NSmax,IFPR
200   FORMAT(/,1x,
     +'Convergence tolerencee (Bathe, Eqs.11.83 and 84)=',E11.5,/,1X,
     +'Limit on number of sweeps= ',I2,/,1X,
     +'Flag =',I2,' Integer 1 requires printing during iteration.')

c     Elements of {Q} at which real arrays start are:
c     [S]:N1  ;  [Am]:N2  ; {EIGV}:N3 ; [X]:N4 ; {D}:N5 
      

      N1=1
      N2= N*N+ N1
      N3= N*N+ N2
      N4= N+N3
      N5= N*N+N4

c     Element of {NI} at which integer array start is:
c      {NEIGV}:K1 

      K1=1

c     CALL DATA(N,S, Am)
      CALL DATA(N,Q(N1),Q(N2))


c     CALL JACOBI(S,AM,X,EIGV,D,N,RTOL,NSmax,IFPR)
      CALL JACOBI(Q(N1),Q(N2),Q(N4),Q(N3),Q(N5),N,RTOL,NSmax,IFPR)



c     CALL RESULTS(A,X,N)
      CALL RESULTS(Q(N3),Q(N4),N)
    

      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATA(N,S,Am)
c     Reading input matrices [S], [Am] 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(N,N),Am(N,N)
      WRITE(6,100)
100   FORMAT(/,1X,'The given matrices:',/)

      WRITE(6,300)
      DO 10 I=1,N
      READ(5,*)(S(I,J),J=1,N)
10    WRITE(6,500)(S(I,J),J=1,N)
300   FORMAT(1X,'Matrix [S] (e. g. stiffness matrix)',/)

500   FORMAT(1X,6(E11.5,1X))

      WRITE(6,200)
      DO 20 I=1,N
      READ(5,*)(Am(I,J),J=1,N)
20    WRITE(6,600)(Am(I,J),J=1,n)
200   FORMAT(/,1X,'Matrix [m] (e. g. mass matrix)',/)
600   FORMAT(1X,6(E11.5,1X))

      WRITE(6,800)
800   FORMAT(/,
     +' ________________________________________________________________
     +',/)

      RETURN
      END



      SUBROUTINE RESULTS(A,X,N)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N),X(N,N)
      DIMENSION IA(N),EV(N),EVEC(N,N)


      CALL Evalues(A,N,IA,EV)

      CALL Evectors(X,N,IA,Evec)

       WRITE(6,400)
400   FORMAT(/,1x,'Summary of results:',/)
      WRITE(6,100)(EV(I),I=1,N)
100   FORMAT(/,1X,'Eigenvalues:',/,/,6(1X,E11.5))
      WRITE(6,300)     
300   FORMAT(/,1X,'Eigenvectors:',/)
      DO 10 I=1,N
      WRITE(6,200)(EVEC(I,J),J=1,N)
200   FORMAT(6(1X,E11.5))
10    WRITE(6,201)
201   FORMAT(/)

      RETURN
      END


      SUBROUTINE Evalues(A,N,IA,B)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N),IA(N),B(N)
     

      DO 19 I=1,N
19    B(I)=0.
      DO 18 I=1,N
18    IA(I)=0

      
      DO 6 K=1,N
      ST=-1.e30

      DO 5 J=1,N

      IF(A(J)-ST)5,5,15
15    ST=A(J)
      JJ=J
5     CONTINUE

      B(N-K+1)=A(JJ)
      IA(N-K+1)=JJ
      A(JJ)=0.

6     CONTINUE
      DO 7 L=1,N
7     IF(ABS(A(L)).GT.0.) B(1)=A(L)

      RETURN
      END


      SUBROUTINE Evectors(X,N,IA,Evec)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION X(N,N),IA(N),Evec(N,N)

      DO 6 J=1,N
      K=IA(J)
      DO 5 I=1,N
      
      Evec(I,J)=X(I,K)
5     CONTINUE
      
6     CONTINUE

      RETURN
      END


c-----------------------------------------------------------
c  Subroutine JACOBI is given in: Bathe, K-J, Finite Element
c  Procedures, Printice Hall, 1996, pp 924-927.     
c----------------------------------------------------------











