

c                        Computer Program INVERT
c                             January, 2016 

c			     by A. Ghali
c		       Professor of Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c			    Matrix inversion
c  Based on Ghali,A., Neville,A. M., Structural Analysis: A Unified Classical
c  and Matrix Approach, CRC Press, Spon Press, 7th edition, 2016, Section A-9 
                      
c      Invert  [A] and store result in [B]
c      Dimensions:   [A]nn   ;[B]nn

c      Input data instructions:

c      The input file must be named INVERT.IN . Give n in one line.
c      Give rows of [A], starting a new line for each row.

c      To run the program, type 'INVERT' and press enter. The result will
c      be in a file named INVERT.OUT .
c__________________________________________________________________________
                          
c              Listing of program INVERSE

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXQ=20000)
      COMMON/REALV/Q(MAXQ)

      CHARACTER * 76 TITLE

      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='INVERT.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='INVERT.OUT')
      DO 70 I=1,MAXQ
70    Q(I)=0.0

      READ(5,101)TITLE
101   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(1X,A76)

      READ(5,*)N
      WRITE(6,100)N,N
100   FORMAT(3X,
     +	  'Inversion of:[A] ; dimension: A(',
     +	   I2,',',I2,')')
      M=N
c     Elements of {Q} at which real arrays start are:
c     [A]:N1  [B]:N2
      N1=1
      N2=N1+N*N
      CALL DATA(Q(N1),Q(N2),N,M)
      CALL SOLN(Q(N1),Q(N2),N,M)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATA(A,B,N,M)
c     Reading input matrice
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,N),B(N,M)
      WRITE(6,100)
100   FORMAT(/,3X,'Matrix to be inverted:')
      DO 10 I=1,N
      READ(5,*)(A(I,J),J=1,N)
10    WRITE(6,300)(A(I,J),J=1,N)
300   FORMAT(3X,2X,5(E13.5,1X))
c     Generation of an identity matrix
      DO 20 I=1,N
20    B(I,I)=1.0
      RETURN
      END

      SUBROUTINE SOLN(A,B,N,M)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,N),B(N,M)
c     Generation of auxiliary matrix and store it in [A] and [B]
c     First row of auxiliary matrix
      DO 10 J=2,N
10    A(1,J)=A(1,J)/A(1,1)
      DO 20 J=1,M
20    B(1,J)=B(1,J)/A(1,1)
c     Remaining rows of auxiliary matrix
      DO 30 I=2,N
      DO 30 J=1,N
      IF(J.GT.I)GO TO 36
      SUM=0
      DO 35 K=1,J-1
35    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=A(I,J)-SUM
      GO TO 38
36    SUM=0
      DO 37 K=1,I-1
37    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=(A(I,J)-SUM)/A(I,I)
38    CONTINUE
30    CONTINUE
      DO 40 I=2,N
      DO 40 J=1,M
      SUM=0
      DO 45 K=1,I-1
45    SUM=SUM+A(I,K)*B(K,J)
      B(I,J)=(B(I,J)-SUM)/A(I,I)
40    CONTINUE
c     Back substitution
      DO 50 L=1,M
      B(N,L)=B(N,L)
      DO 50 I=2,N
      IB=N-I+1
      SUM=0
      DO 55 K=IB+1,N
55    SUM=SUM+A(IB,K)*B(K,L)
50    B(IB,L)=B(IB,L)-SUM
      WRITE(6,400)
400   FORMAT(/,3X,'The inverted matrix :' )
      DO 60 I=1,N
60    WRITE(6,500)(B(I,J),J=1,M)
500   FORMAT(3X,2X,5(E13.5,1X))
      RETURN
      END
