
c                        Computer Program SOLVE
c                             January 2016

c			     by A. Ghali
c		       Professor of Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c                     Solution of simultaneous equations
c  Based on Ghali,A., Neville,A. M., Structural Analysis :A Unified Classical
c  and Matrix Approach, CRC Press, Spon  Press, 7th ed. 2016, Section A-9

c     Solve:  [A][X]=[B]
c     Dimensions:   [A]nn   ;[X]nm ; [B]nm
  
c     Input data instuctions:

c     Give n, m in one line. Give in one line first row of of [A], followed
c     by first row of[B]. Do the same for the remaining rows.
  
c     To run the program, type 'SOLVE' and press enter. The result will
c     be in a file named SOLVE.OUT. [A] can be non-symmetric.
c___________________________________________________________________________


c      Solve:  [A][X]=[B]
c      Dimensions:   [A]nn   ;[X]nm ; [B]nm


c		 Listing of program SOLVE

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXQ=20000)
      COMMON/REALV/Q(MAXQ)

      CHARACTER * 76 TITLE

      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='SOLVE.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='SOLVE.OUT')
      DO 70 I=1,MAXQ
70    Q(I)=0.0

      READ(5,101)TITLE
101   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(1X,A76)

      READ(5,*)N,M
      WRITE(6,100)N,N,N,M
100   FORMAT(3X,
     +	  'Solution of: [A][X]=[B] ; dimensions: A('I2,',',
     +	  I2,') and B:(',I2,',',I2,')')
c     Elements of {Q} at which real arrays start are:
c     [A]:N1  [B]:N2
      N1=1
      N2=N1+N*N
      CALL DATA(Q(N1),Q(N2),N,M)
      CALL SOLN(Q(N1),Q(N2),N,M)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATA(A,B,N,M)
c     Reading input matrices
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,N),B(N,M)
      WRITE(6,100)
100   FORMAT(/,3X,'Equations to be solved:')
      DO 10 I=1,N
      READ(5,*)(A(I,J),J=1,N),(B(I,J),J=1,M)
10    WRITE(6,300)(A(I,J),J=1,N),(B(I,J),J=1,M)
300   FORMAT(3X,2X,5(E13.5,1X))
      RETURN
      END

      SUBROUTINE SOLN(A,B,N,M)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION A(N,N),B(N,M)
c     Generation of auxiliary matrix and store it in [A] and [B]
c     First row of auxiliary matrix
      DO 10 J=2,N
10    A(1,J)=A(1,J)/A(1,1)
      DO 20 J=1,M
20    B(1,J)=B(1,J)/A(1,1)
c     Remaining rows of auxiliary matrix
      DO 30 I=2,N
      DO 30 J=2,N
      IF(J.GT.I)GO TO 36
      SUM=0
      DO 35 K=1,J-1
35    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=A(I,J)-SUM
      GO TO 38
36    SUM=0
      DO 37 K=1,I-1
37    SUM=SUM+A(I,K)*A(K,J)
      A(I,J)=(A(I,J)-SUM)/A(I,I)
38    CONTINUE
30    CONTINUE
      DO 40 I=2,N
      DO 40 J=1,M
      SUM=0
      DO 45 K=1,I-1
45    SUM=SUM+A(I,K)*B(K,J)
      B(I,J)=(B(I,J)-SUM)/A(I,I)
40    CONTINUE
c     Back substitution; store [X] in [B].
      DO 50 L=1,M
      B(N,L)=B(N,L)
      DO 50 I=2,N
      IB=N-I+1
      SUM=0
      DO 55 K=IB+1,N
55    SUM=SUM+A(IB,K)*B(K,L)
50    B(IB,L)=B(IB,L)-SUM
      WRITE(6,400)
400   FORMAT(/,3X,'Matrix [X]:' )
      DO 60 I=1,N
60    WRITE(6,500)(B(I,J),J=1,M)
500   FORMAT(3X,2X,5(E13.5,1X))
      RETURN
      END
