c			 Computer Program NLPF
c                             January 2016

c			     by A. Ghali
c		       Professor of Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c           Geometrically Nonlinearl Analysis of Plane Frames
c Basis: Ghali,A. and Neville,A. M., Structural Analysis :A Unified Classical
c and Matrix Approach, CRC Press, Spon Press, 7th ed.,2016, Chapter 25.        Chapman and Hall,London and New York,4th ed.,1997,Chapter 24
c------------------------------------------------------------------------------
c				 Scope
c    Program NLPF analyses frames with straight prismatic members ,for which
c  the centroidal axes are in the xy plane. Axial and bending deformations are
c  considered. The analysis gives changes in nodal displacements, reactions
c  and member end forces. Geometrical nonlinearity is considerd; this means
c  that the equilibrium of nodes and members is satisfied in the displaced
c  position of the nodes and the deformed configuration of members.
c    Initially the structure may be subjected to prestressing, temperature
c  changes or nodal forces. Unlike linear analysis, the member end forces in
c  initial stage must be included in input data. The x and y coordinates of
c  the nodes in the input data represent position of the nodes existing before
c  application of new loads for which the changes of displacements and member
c  end forces are required.
c    The new loading can be any, or cmbination,of the following types:
c  (a) Two concentrated forces and a couple:Fx, Fy and Mz at a node.
c  (b) Temperature, shrinkage etc. in idividual members; in which case
c      three member fixed-end forces ,{Ar} must be included in the input;
c      these are axial force and two end moments for each member. The
c      computer considers the remaining three fixed-end forces as the
c      equilibrants of the three given forces.
c
c			      Sign Convention
c    Positive directions of nodal displacements u, v and theta and  member
c  end forces are defined in Figs. 22-1 and 22-2 of Ghali and Neville. The
c  reaction components are determined in the directions of the global axes.
c
c			  Input Data Instructions
c    The data, in free format, must be in a file named NLPF.IN, onsisting of:
c  - NJ,NM,NSJ,NSTG,NITER,TOLER Number of joints, members and joints with
c    prescribed displacement(s); number of load stages; maximum number of
c    iteration in each  stage;TOLER is the tolerence value, defined as:

c      TOLER=SQRT({g}T {g})/ SQRT({F}T	{F})

c      {g} and {F} are the out-of-balance forces and the nodal forces (total
c      introduced at all stages).

c  - Load-stage vector of real numbers, {STEPS}. The number of elements in
c      the vector is equal to the number of load stages. The external applied
c      nodal forces (load type a) and {Ar} (load type b) will be introduced in
c      stages, using  the multipliers in this vector.

c  - Single line of one real value for E .

c  - A set of lines ;their number equals number of nodes.Each line is compsed
c    of a node number followed by its x and y coordinates.

c  - A set of lines their number equals number of members.Each line starts
c    with 3 integers: a member number ,its starting node and its end node,
c    followed by two real values : a and I ; ar is not included because
c      shear deformations are ignored.
c  - A set of lines their number equals number of joints with prescribed
c    displacements. Each line starts with a node number and three restraint
c    indicators (0 or 1),followed by three real values of prescribed
c    displacements. The restraint indicators 1 and 0 mean free and prescribed
c    displacement component,respectively. When a displacement component is
c    free, a dummy value must be filled in for the prescribed displacement
c    (any real number)

c  - A set of any number of lines for loading of type(a).Each line is comp-
c      osed of an integer , a node number ,followed by three real values of
c      nodal forces Fx,Fy and Mz.The last line of the set must start by an
c      greater than the number of nodes follwed by threee dummy real values.
c      The last line must not be omitted even when there is no loading of
c      type (a).

c  - A set of any number of lines for loading of type(b).Each line is
c    composed of an integer, which is a member number, followed by three real
c    values:an axial force and two end moments at the member's first and
c    second node,respectively.  These are fixed-end forces caused by temp-
c    erature, shrinkage, etc. ; tensile axial force and clockwise end moments
c    are positive. The last line of this set must start with an  integer
c    greater than the number of members followed by three dummy real values.
c    This line must not be omitted even when there is no loading of type(b).

c  - A set of any number of lines giving initial prestressing member end
c    forces. Each line is composed of member number followed by three
c    real values equal to the axial force (positive when tensile) followed
c    by the end moments at 1st and second node (positive when clockwise)
c    The last line of the set starts with a number greater than number of
c    members, followed by three dummy real values. This line must not be
c    omitted even when there is no initial prestressing.

c    The load-stage vector introduces the loads of types (a) and (b) in
c  stages. The prescribed displacements are also introduced in stages using
c  the same fractions in the load-stage vector. Newton-Raphson's iterations
c  ations are performed in each stage to eliminate the out-of-balance forces
c  before proceeding to the next stage. A support conditions indicator=1 for
c  any coordinate will mean free displacement in all load stages. The initial
c  forces affect the geometrical stiffness of the members; but the displacements
c  which have occured when the initial prestress is introduced are not the
c  objective of this analysis. Thus the {x} and {y} coordinates of the nodes
c  represent the position of the nodes in the prestressed structure.

c    Examle input data:

c Example 24-4. Column with vl. and hl. forces  at tip. 
c  2	1    1	  1    6    1.e-8      NJ,NM,NSJ,NSTG,NITER,TOLER
c  1.0                                 NSTG- load multipliers 
c  10.e6			       Elasticity modulus

c  1	     0.0	  .0	       Node number, x and y coordinates
c  2	    75.0	  .0

c  1	1    2	  1.	  .1	       Member number, JS, JE, cross-sec. a, I
c  2	0   0	0     0.   0.	0.     Node no.,restr. indics.,prsc. displs.

c  1  150.  150.   .0		       Node, Fx, Fy and Mz

c 100	3*0.			       Dummy,end of data of loaded nodes

c 100	3*0.			       Dummy,end of data for {Ar}

c 100	3*0.			       Dummy, end initial prestress data {AINIT}

c  To run the program , type NLPF and press Enter. The program will execute
c  the analysis and give the input and the results in a file named NLPF.OUT .

c			     Limits on Data
c    Vectors {B} and {NI} are used to store all the real and the integer arrays
c  used by the program. The maximum dimensions set are: MAXB=20000 and MAXNI=
c  2000 . This would normally be sufficient for a frame approximately 120 nodes,
c  120 members, depending upon the band width. The maximum dimensions may be
c  changed as required on line 2 of program(see listing below). After any
c  change, the program must be compiled, using a Fortran compiler(e.g. Microsoft
c  Fortran compiler(e.g. Microsoft Fortran Otimizing compiler).

c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  {A}	  cross-section areas					     NM
c  {AI}   second moments of areas of cross-sections		     NM
c  {AINIT} initial member end forces due to prestressig 	     3*NM
c  [AR]   member end forces with nodal displacements restrained      3*NM
c  {AUPDAT} updated member end forces				     3*NM
c  {D}	  nodal displacement increments 			     3*NJ
c  {DUPDAT} updated nodal displacements
c  E	  modulus of elasticity
c  [ES]   element (member) elastic stiffness in local coordinates    6	    6
c  [EST]  element elastic stiffness matrix in directions parallel    6	    6
c	  to global axes
c  [GS]   element (member) gemetric stiffness with respect to	     6	    6
c	  global axes
c  {F}	  nodal restraining forces				    3*NJ
c  {FINIT} initial nodal forces equilibrating {AINIT} included
c	   in input data.
c  {FUPDAT} part of {F} intoduced in current and earlier	    3*NJ
c	  load stages.
c  [ISUP] joint restraint indicators : 1 and 0 ,indicating free      NSJ    3
c	  and prescribed displacement respectively
c  {JE}   end nodes of individual members			     NM
c  {JS}   starting node of individual members			     NM
c  NB	  band width ; see Sec. 24-3 of Ghali and Neville
c  NITER  maximum number of iterations in each load stage
c  NJ	  number of joints (nodes)
c  NM	  number of members
c  NSTG   number of load stages
c  {NS}   list of node numbers where supports are provided or	    NSJ
c	  displacements are prescribed
c  NSJ	  number of supprted joints (nodes with prescribed displac-
c	  ements)
c {OOBF}  out-of-balance forces 				    3*NJ
c {PVALUE} axial forces in  members				    NM
c  [PD]   total prescribed displacement values introduced in all    NSJ   3
c	  stages
c [PDUPDAT] the part of [PD] intoduced in current and earlier	    NSJ   3
c	  load stages.
c  [R]	  reactions at the three coordinates of supported nodes     NSJ    3
c  [RSAVE] reactions due to forces applied at the nodes 	    NSJ    3
c  SUMOOBF (sum square of out-of-balance forces)**0.5
c  SUMF   (sum square of nodal forces,excluding reactions)**0.5
c  SUMR   (sum square of reaction components)**0.5
c  [S]	  tangent stiffness matrix (upper triangle only)	    3*NJ   NB
c  {STEPS} fractions of load to be introduced in each stage	    NSTG
c  TOLER  tolerence ratio
c  {X}	  x coordinate of nodes before displacements		    NJ
c  {XUPDAT} updated x coordinates (after displacements) 	    NJ
c  {Y}	  y coordinate of nodes before displacements		    NJ
c  {YUPDAT} updated y coordinates (after displacements) 	    NJ
c------------------------------------------------------------------------------

c		 Listing of program NLPF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='NLPF.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='NLPF.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)
      READ(5,*)NJ,NM,NSJ,NSTG,NITER,TOLER
      WRITE(6,300)NJ
300   FORMAT(3X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(3X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(3X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NSTG
600   FORMAT(3X,'Number of load stages=',I5)
      WRITE(6,650)NITER
650   FORMAT(3X,'Maximum number of iterations in each load stage=',I3)
      WRITE(6,750)TOLER
750   FORMAT(3X,'Tolerence=',E13.5,/,3X,
     +'The tolerence is the allowed error. The error is square root of',
     +/,3X,
     +'sum of squares of out-of-balance forces divided by square root ',
     +/,3X,
     +'of sum of squares of nodal forces {F}. When {F}={0}, the error is
     +',/,3X,
     +'sum of squares of out-of-balance forces divided by square root ',
     +/,3X,
     +'of sum of squares of reaction components.')
      NJ3=3*NJ
      NS3=3*NSJ
c     Elements of {B} at which real arrays start are:
c     {A}:N1  {AI}:N2  {AINIT}:N3  {AR}:N4   {D}:N5  {F}:N6  {FSTEP}:N7
c     [PD]:N8  {R}:N9  {STEPS}:N10  {X}:N11  {Y}:N12	[S]:N13
c     {AUPDAT}:N14  {DUPDAT}:N15  {XUPDAT}:N16	{YUPDAT}:N17 {FUPDAT}:N18
c     {OOBF}:N19    [PDUPDAT]:N20  {PVALUE}:N21 {FINIT}:N22  RSAVE:N23
      N1=1
      N2=N1+NM
      N3=N2+NM
      N4=N3+3*NM
      N5=N4+3*NM
      N6=N5+NJ3
      N7=N6+NJ3
      N8=N7+NJ3
      N9=N8+NS3
      N10=N9+NS3
      N11=N10+NSTG
      N12=N11+NJ
      N13=N12+NJ
c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4
      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      NIUSED=K4+NS3
c     CALL DATAPF(X,Y,JS,JE,A,AI,NS,ISUP,PD,NB,NSTG,STEPS)
      CALL DATAPF(B(N11),B(N12),NI(K1),NI(K2),B(N1),B(N2),NI(K3),
     *NI(K4),B(N8),NB,NSTG,B(N10) )
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N14=N13+NJ3*NB
      N15=N14+3*NM
      N16=N15+NJ3
      N17=N16+NJ
      N18=N17+NJ
      N19=N18+NJ3
      N20=N19+NJ3
      N21=N20+NS3
      N22=N21+NM
      N23=N22+NJ3
      NBUSED=N23+NS3
      WRITE(6,755)NBUSED,NIUSED
755   FORMAT(/,3X
     +	'This problem requires a vector of',I5,' real values.',
     +/,3X,'It also requies a vector of',I4,' integers.')
c     CALL LOADPF(JS,JE,X,Y,F,AR,NJ3,AINIT,FINIT,NS,RSAVE)
      CALL LOADPF(NI(K1),NI(K2),B(N11),B(N12),B(N6),B(N4),NJ3,B(N3),
     + B(N22),NI(K3),B(N23))
c     CALL STARTX_Y(XUPDAT,YUPDAT,X,Y)
      CALL STARTX_Y(B(N16),B(N17),B(N11),B(N12))
c     CALL STARTA(AUPDAT,AINIT)
      CALL STARTA(B(N14),B(N3))
      DO 999 NSTEP=1,NSTG
      WRITE(6,150)NSTEP
150   FORMAT(3X,'Load stage number ',I2)
c     CALL ADDAR(AUPDAT,AR,STEPS,NSTEP,NSTG,FACTOR)
      CALL ADDAR(B(N14),B(N4),B(N10),NSTEP,NSTG,FACTOR)
      WRITE(6,145)FACTOR
145   FORMAT(3X,'Factor representing load level reached at this stage=',
     +E15.5)
c     CALL UPDATEF(NJ3,F,FUPDAT,STEPS,NSTG,NSTEP,PD,PDUPDAT,SUMF,
c    + NS,ISUP)
      CALL UPDATEF(NJ3,B(N6),B(N18),B(N10),NSTG,NSTEP,B(N8),B(N20),SUMF,
     +	NI(K3),NI(K4))
      IREFINE=0
      ITER=0
8     CONTINUE
c     The following 2 lines has been moved below near statement 995(Dec. 2000).
c     CALL UPDATEX_Y(XUPDAT,YUPDAT,D,NJ3)
c     CALL UPDATEX_Y(B(N16),B(N17),B(N5),NJ3)
c     CALL OOBFS(JS,JE,XUPDAT,YUPDAT,FUPDAT,AUPDAT,NJ3,OOBF,
c    +	SUMOOBF, NS,ISUP,SUMR,R,FINIT,RSAVE,NSTEP,STEPS,NSTG,AR,X,Y)
      CALL OOBFS(NI(K1),NI(K2),B(N16),B(N17),B(N18),B(N14),NJ3,B(N19),
     + SUMOOBF,NI(K3),NI(K4),SUMR,B(N9),B(N22),B(N23),NSTEP,B(N10),NSTG,
     + B(N4),B(N11),B(N12))
c     IF(ITER.LE.1) GO TO 11  This statement has been revised on the
c     following statement (Dec. 2000).
      IF(ITER.LT.1) GO TO 11
      IF(SUMF.EQ.0.0)ERROR=SUMOOBF/SUMR
      IF(SUMF.NE.0.0)ERROR=SUMOOBF/SUMF
      IF(ERROR.LE.TOLER)GO TO 995
11    CONTINUE
      ITER=ITER+1
      IF(ITER.GT.NITER)GO TO 995
c     WRITE(6,350)ITER
350   FORMAT(3X,'Iteration number ',I2)
c     CALL STIFPF(JS,JE,A,AI,XUPDAT,YUPDAT,S,NJ3,NB,AUPDAT,X,Y,IREFINE,
c    +	 PVALUE,AR,FACTOR,ITER)
      CALL STIFPF(NI(K1),NI(K2),B(N1),B(N2),B(N16),B(N17),B(N13),
     *NJ3,NB,B(N14),B(N11),B(N12),IREFINE,B(N21),B(N4),FACTOR,ITER)
c     CALL SUPTPF(S,OOBF,NS,ISUP,PDUPDAT,NJ3,NB,ITER,IREFINE)
      CALL SUPTPF(B(N13),B(N19),NI(K3),NI(K4),B(N20),NJ3,NB,
     +	ITER,IREFINE)
c     CALL EQSOLN(S,OOBF,D,NJ3,NB,1)
      CALL EQSOLN(B(N13),B(N19),B(N5),NJ3,NB,1)
c     CALL UPDATEA(D,XUPDAT,YUPDAT,JS,JE,A,AI,AUPDAT,NJ3,X,Y,
c    + IREFINE,PVALUE)
      CALL UPDATEA(B(N5),B(N16),B(N17),NI(K1),NI(K2),B(N1),B(N2),B(N14),
     +	 NJ3, B(N11),B(N12),IREFINE,B(N21))
c     CALL UPDATED(DUPDAT,D,NJ3)
      CALL UPDATED(B(N15),B(N5),NJ3 )

c     The following 2 lines (Dec.,2000) have been moved from immediately below statement 8
c    +below statement 8.
c     CALL UPDATEX_Y(XUPDAT,YUPDAT,D,NJ3)
      CALL UPDATEX_Y(B(N16),B(N17),B(N5),NJ3)
      GO TO 8
995   CONTINUE
c     CALL RSLTPF(DUPDAT,NS,R,AUPDAT,NJ3,ITER,ERROR,IREFINE,
c    + X,Y,JS,JE)
      CALL RSLTPF(B(N15),NI(K3),B(N9),B(N14),NJ3,ITER,ERROR,IREFINE,
     + B(N11),B(N12),NI(K1),NI(K2))
c     The results obtained above are inaccurate because the values of S and
c     t used in calculating the member end forces are not based on the final
c     values of the axial forces. The following iterations are to improve the
c     accuracy. The values of axial forces obtained above will now be treated
c     as constants in the refinement iterations.
c     Store the axial forces in {PVALUE}.
      WRITE(6,87)
87    FORMAT(3X,
     + 'Refinement of results:'
     +,/,3X,
     +'Out-of-balance forces are determined considering the effects of'
     +,/,3X,
     +'axial forces on member end rotation stiffnesses. This yields out-
     +',/,3X,
     +'of-balance forces, eliminated by refinement iteration cycles in'
     +,/,3X,
     +'which constant set of axial forces is employed. The refinement '
     +,/,3X,
     +'cycles are repeated until square root sum of squares of axial'
     +,/,3X,
     +'forces in two consecutive trial axial-force sets differ by less'
     +,/,3X,
     +'than 0.01 percent or, when the total number of axial-force sets'
     +,/,3X,
     +'employed reaches 8.')
      NPCYCLE=1
81    CONTINUE
c     The results obtained above are inaccurate because the values of S and
c     t used in calculating the member end forces are not based on the final
c     values of the axial forces. The following iterations are to improve the
c     accuracy. The values of axial forces obtained above will now be treated
c     as constants in the refinement iterations.
c     Store the axial forces in {PVALUE}.
c     CALL PVALUES(AUPDAT,PVALUE)
      CALL PVALUES(B(N14),B(N21))
      SUMPOLD=SUMP
      SUMP=0.
      DO 82 LL=1,NM
82    SUMP=SUMP+B(N21-1+LL)*B(N21-1+LL)
      SUMP=SQRT(SUMP)
      IF(IREFINE.EQ.0)GO TO 79
      IF(NPCYCLE.LT.1)GO TO 78
      IF(SUMPOLD.LT. .0001)GO TO 83
      IF(ABS( (SUMP-SUMPOLD)/SUMPOLD) .LT. 1.e-4)GO TO 83
78    CONTINUE
79    IREFINE=1
      NPCYCLE=NPCYCLE+1
      ITER=0
c     Calculate the member end forces from the latest geometry, not
c     as summation of the increments in each iteration. For this purpose,
c     set {AUPDAT}={AINIT}+COOF*{Ar} in the subroutine SETA below.
c     CALL SETA(AUPDAT,AINIT,AR,STEPS,NSTEP,NSTG)
      CALL SETA(B(N14),B(N3),B(N4),B(N10),NSTEP,NSTG)
c     CALL VERIFYA(DUPDAT,X,Y,JS,JE,A,AI,AUPDAT,NJ3,
c    +	PVALUE)
      CALL VERIFYA(B(N15),B(N11),B(N12),NI(K1),NI(K2),B(N1),B(N2),
     +	B(N14),NJ3,B(N21))
88    CONTINUE
c     IF(ITER.GT.0)CALL UPDATEX_Y(XUPDAT,YUPDAT,D,NJ3)
      IF(ITER.GT.0)CALL UPDATEX_Y(B(N16),B(N17),B(N5),NJ3)
c     CALL OOBFS(JS,JE,XUPDAT,YUPDAT,FUPDAT,AUPDAT,NJ3,OOBF,SUMOOBF,
c    +	 NS,ISUP,SUMR,R,FINIT,RSAVE,NSTEP,STEPS,NSTG,AR,X,Y)
      CALL OOBFS(NI(K1),NI(K2),B(N16),B(N17),B(N18),B(N14),NJ3,B(N19),
     + SUMOOBF,NI(K3),NI(K4),SUMR,B(N9),B(N22),B(N23),NSTEP,B(N10),NSTG,
     + B(N4),B(N11),B(N12))
      IF(SUMF.EQ.0.0)ERROR=SUMOOBF/SUMR
      IF(SUMF.NE.0.0)ERROR=SUMOOBF/SUMF
      IF(ERROR.LE.TOLER)GO TO 990
c     CONTINUE
      ITER=ITER+1
      IF(ITER.GT.NITER)GO TO 990
c     WRITE(6,350)ITER
c     CALL STIFPF(JS,JE,A,AI,XUPDAT,YUPDAT,S,NJ3,NB,AUPDAT,X,Y,IREFINE,
c    +	  PVALUE,AR,FACTOR,ITER)
      CALL STIFPF(NI(K1),NI(K2),B(N1),B(N2),B(N16),B(N17),B(N13),
     *NJ3,NB,B(N14),B(N11),B(N12),IREFINE,B(N21),B(N4),FACTOR,ITER)
c     CALL SUPTPF(S,OOBF,NS,ISUP,PDUPDAT,NJ3,NB,ITER,IREFINE)
      CALL SUPTPF(B(N13),B(N19),NI(K3),NI(K4),B(N20),NJ3,NB,
     +	  ITER,IREFINE)
c     CALL EQSOLN(S,OOBF,D,NJ3,NB,1)
      CALL EQSOLN(B(N13),B(N19),B(N5),NJ3,NB,1)
c     Calculate updated member end forces by adding the change in current
c     iteration.
c     CALL UPDATEA(D,XUPDAT,YUPDAT,JS,JE,A,AI,AUPDAT,NJ3,X,Y,
c    + IREFINE,PVALUE)
      CALL UPDATEA(B(N5),B(N16),B(N17),NI(K1),NI(K2),B(N1),B(N2),B(N14),
     +	 NJ3, B(N11),B(N12),IREFINE,B(N21))
c     CALL UPDATED(DUPDAT,D,NJ3)
      CALL UPDATED(B(N15),B(N5),NJ3 )
      GO TO 88
990   CONTINUE
      WRITE(6,877)(NPCYCLE-1),ITER
877   FORMAT(/,3X,'Set of axial forces number=',I2,/,3X,
     +	    'Number of iterations in which this set is employed=',I2)
      IF(NPCYCLE.GT.8)GO TO 83
      GO TO 81
83    CONTINUE
      IF(ITER.GT.0)WRITE(6,84)(NPCYCLE-1)
84    FORMAT(/,3X,'Total number of sets of axial forces employed=',I2)
c     CALL RSLTPF(DUPDAT,NS,R,AUPDAT,NJ3,ITER,ERROR,IREFINE,
c    + X,Y,JS,JE)
      CALL RSLTPF(B(N15),NI(K3),B(N9),B(N14),NJ3,ITER,ERROR,IREFINE,
     + B(N11),B(N12),NI(K1),NI(K2))
999   CONTINUE
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATAPF(X,Y,JS,JE,A,AI,NS,ISUP,PD,NB,NSTG,STEPS)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION X(NJ),Y(NJ),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3),STEPS(NSTG)
c     Read the load fractions to be introduced in the load stages.
      READ(5,*)(STEPS(I),I=1,NSTG)
      WRITE(6,640)(STEPS(I),I=1,NSTG)
640   FORMAT(/,3X,'Load fractions in different stages:',/,3X,
     +10(F6.4,1X))
      READ(5,*)E
      WRITE(6,200)E
200   FORMAT(/,3X,'Elasticity modulus=',E13.5)
      WRITE(6,300)
300   FORMAT(/,3X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',5x,'x',9X,'y')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J)
10    WRITE(6,500)J,X(J),Y(J)
500   FORMAT(3X,I5,2F10.3)
      WRITE(6,600)
600   FORMAT(/,3X,'Element information')
      WRITE(6,700)
700   FORMAT(3X,'Element',2X,'1st node',3X,'2nd node',6x,'a',13x,'I',
     *13x)
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),A(J),AI(J)
c     Member no.,starting node,end node,cross-section area,2nd moment
c     of area.
20    WRITE(6,800)J,JS(J),JE(J),A(J),AI(J)
800   FORMAT(3X,I5,4X,I5,5X,I5,3X,2(E13.5,1X))
      WRITE(6,900)
900   FORMAT(/,3X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',5X,'Restraint indicators',7X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or o,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'v',3X,'theta',8x,'u',13x,'v',12x,'theta')
1200  FORMAT(3X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 22-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STARTA(AUPDAT,AINIT)
c     Set the member end forces equal to the initial presressing values.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION AUPDAT(3*NM),AINIT(3*NM)
      DO 5 I=1, (3*NM)
      AUPDAT(I)=AINIT(I)
5     CONTINUE
      RETURN
      END

      SUBROUTINE ADDAR(AUPDAT,AR,STEPS,NSTEP,NSTG,FACTOR)
c     Add the fraction of the restraining end forces {Ar} introduced in
c     the current load stage to the member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION AUPDAT(3*NM),AR(3*NM),STEPS(NSTG)
      COOF=STEPS(NSTEP)
      DO 5 I=1, (3*NM)
      AUPDAT(I)=AUPDAT(I)+COOF*AR(I)
5     CONTINUE
      FACTOR=0.
      DO 6 I=1,NSTEP
6     FACTOR=FACTOR+STEPS(I)
      RETURN
      END

      SUBROUTINE SETA(AUPDAT,AINIT,AR,STEPS,NSTEP,NSTG)
c     Set the axial forces and the end moments in the members equal to the
c     initial values plus the part of {AR} introduced up to and including
c     the current load stage.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION AUPDAT(3*NM),AR(3*NM),STEPS(NSTG)
      DIMENSION AINIT(3*NM)
      COOF=0.
      DO 2 I=1,NSTEP
2     COOF=COOF+STEPS(I)
      DO 5 I=1,NM
      AUPDAT(3*I-2)=AINIT(3*I-2)+COOF*AR(3*I-2)
      AUPDAT(3*I-1)=AINIT(3*I-1)+COOF*AR(3*I-1)
      AUPDAT(3*I)=AINIT(3*I)+COOF*AR(3*I)
5     CONTINUE
      RETURN
      END

      SUBROUTINE STARTX_Y(XUPDAT,YUPDAT,X,Y)
c     Start the analysis by setting the updated coordinates of the nodes
c     equal to the given data values.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION X(NJ),Y(NJ),XUPDAT(NJ),YUPDAT(NJ)
      DO 6 I=1,NJ
      XUPDAT(I)=X(I)
      YUPDAT(I)=Y(I)
6     CONTINUE
      RETURN
      END

      SUBROUTINE STIFPF(JS,JE,A,AI,XUPDAT,YUPDAT,S,NJ3,NB,AUPDAT,X,Y,
     +	IREFINE,PVALUE,AR,FACTOR,ITER)
c     Stiffness matrix for plane frame.See Sec. 24-9,Ghali and Neville
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION JS(NM),JE(NM),A(NM),AI(NM),XUPDAT(NJ),YUPDAT(NJ)
      DIMENSION S(NJ3,NB),AUPDAT(3*NM),X(NJ),Y(NJ)
      DIMENSION ES(6,6),EST(6,6),IM(6),PVALUE(NM),GS(6,6)
      DIMENSION AR(3*NM)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 50 JM=1,NM
      ALX=XUPDAT(JE(JM))-XUPDAT(JS(JM))
      ALY=YUPDAT(JE(JM))-YUPDAT(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      AJ=A(JM)
      AIJ=AI(JM)
c     Member end rotational stiffness and carry-over moment.
      ALJINIT=DSQRT((X(JE(JM))-X(JS(JM)))**2+ (Y(JE(JM))-Y(JS(JM)))**2)
      PP=AUPDAT(3*(JM-1)+1)
      IF(IREFINE.EQ.1)PP=PVALUE(JM)
      IF(IREFINE.EQ.1)CALL ENDRS(ALJINIT,AIJ,E,PP,SS,T)
      IF(IREFINE.EQ.0)SS=4.*E*AIJ/ALJINIT
      IF(IREFINE.EQ.0)T=SS/2.
      CALL ESTFPF(ALJ,AJ,E,ES,SS,T,ALJINIT)
c     Transform element stiffness,[ES] into [EST] in global directions;
c     see Eq. 24-27 of Ghali and Neville.
      DO 20 I=1,6
      DO 20 J=1,6
20    EST(I,J)=0.0
      EST(1,1)=ES(1,1)*C*C+ES(2,2)*SIN*SIN
      EST(4,4)=EST(1,1)
      EST(4,1)=-EST(1,1)
      EST(3,3)=ES(3,3)
      EST(6,6)=EST(3,3)
      EST(2,2)=ES(1,1)*SIN*SIN+ES(2,2)*C*C
      EST(5,5)=EST(2,2)
      EST(5,2)=-EST(2,2)
      EST(6,3)=ES(6,3)
      EST(2,1)=(ES(1,1)-ES(2,2))*C*SIN
      EST(5,1)=-EST(2,1)
      EST(5,4)=EST(2,1)
      EST(3,1)=-ES(3,2)*SIN
      EST(4,3)=-EST(3,1)
      EST(6,1)=EST(3,1)
      EST(6,4)=-EST(3,1)
      EST(3,2)=ES(3,2)*C
      EST(5,3)=-EST(3,2)
      EST(4,2)=-EST(2,1)
      EST(6,2)=EST(3,2)
      EST(6,5)=-EST(3,2)
      DO 30 I=1,6
      DO 30 J=I+1,6
30    EST(I,J)=EST(J,I)
      DO 33 I=1,6
      DO 33 J=1,6
33    GS(I,J)=0.
      A1=-AUPDAT(3*JM-2)
      A2=(AUPDAT(3*JM-1)+AUPDAT(3*JM))/ALJ
      IF((ITER.EQ.1).AND.(IREFINE.EQ.0))A1=A1+FACTOR*AR(3*JM-2)
      IF((ITER.EQ.1).AND.(IREFINE.EQ.0))
     +A2=A2-FACTOR*(AR(3*JM-1)+AR(3*JM))/ALJ
c     Add the geometric stiffness matrix to obtain the tangent stiffness
c     matrix of member.
      GS(1,1)=-(A1/ALJ)*(SIN*SIN)
     +	      -(A2/ALJ)*(2.*SIN*C)
      GS(2,1)= -(A1/ALJ)*(-SIN*C)
     +	      -(A2/ALJ)*(SIN*SIN-C*C)
      GS(2,2)= -(A1/ALJ)*(C*C)
     +	      -(A2/ALJ)*(-2.*SIN*C)
      GS(4,1)=-GS(1,1)
      GS(4,2)=-GS(2,1)
      GS(4,4)=GS(1,1)
      GS(5,1)=-GS(2,1)
      GS(5,2)=-GS(2,2)
      GS(5,4)=GS(2,1)
      GS(5,5)=GS(2,2)
      DO 31 I=1,6
      DO 31 J=I+1,6
31    GS(I,J)=GS(J,I)
      DO 32 I=1,6
      DO 32 J=1,6
32    EST(I,J)=EST(I,J)+GS(I,J)
c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END
      SUBROUTINE ESTFPF(ALJ,AJ,E,ES,SS,T,ALJINIT)
c     Element stiffness matrix in local coordinates of a member of plane
c     frame, considering axial force .
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ES(6,6)
      DO 10 I=1,6
      DO 10 J=1,6
10    ES(I,J)=0.0
      ES(1,1)=E*AJ/ALJINIT
      ES(4,1)=-ES(1,1)
      ES(4,4)=ES(1,1)
      ES(2,2)=(2.*(SS+T)/(ALJ*ALJ))
      ES(5,2)=-ES(2,2)
      ES(5,5)=ES(2,2)
      ES(3,3)=SS
      ES(6,6)=ES(3,3)
      ES(3,2)=(SS+T)/ALJ
      ES(6,2)=ES(3,2)
      ES(5,3)=-ES(6,2)
      ES(6,3)=T
      ES(6,5)=-ES(3,2)
      DO 20 I=1,6
      DO 20 J=I+1,6
20    ES(I,J)=ES(J,I)
      RETURN
      END

      SUBROUTINE LOADPF(JS,JE,X,Y,F,AR,NJ3,AINIT,FINIT,NS,RSAVE)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION JS(NM),JE(NM),X(NJ),Y(NJ),F(NJ3)
      DIMENSION AR(3*NM),AINIT(3*NM),FINIT(3*NJ),NS(NSJ),RSAVE(NSJ,3)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of nodes;
c     the line must be completed by 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,3X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(3X,'Node',12X,'Fx',12X,'Fy',12X,'Mz')
20    CONTINUE
      READ(5,*)NODE,FX,FY,AMZ
      WRITE(6,300)NODE,FX,FY,AMZ
300   FORMAT(3X,I5,6X,3(1X,E13.5))
      IF(NODE.GT.NJ)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1)=F(N1)-FX
      F(N2)=F(N2)-FY
      F(N3)=F(N3)-AMZ
c     Save forces applied on the supported nodes to be used in calculation of
c     reactions; see subroutine OOBS.
      DO 21 L=1,NSJ
      IF(NODE.EQ.NS(L))RSAVE(L,1)=-FX
      IF(NODE.EQ.NS(L))RSAVE(L,2)=-FY
      IF(NODE.EQ.NS(L))RSAVE(L,3)=-AMZ
21    CONTINUE
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     due to temperature,etc..Note that the last line in this set of
c     data lines must start with an integer greater than NM;the
c     line must be completed by  three real numbers. The first real number
c     is the axial force (positive when tensile), the second and third real
c     values are the fixed-end moments (positive when clockwise).
c     The positive directions of the end forces are shown in Fig.22-2 of
      WRITE(6,400)
400   FORMAT(/,3X,'Member axial force and end moments with nodal displac
     *ement restrained')
      WRITE(6,500)
500   FORMAT(3X,'Member',2X,'Axial force',2X,'1st node moment',2X,
     *'2nd node moment')
40    CONTINUE
      READ(5,*)K,Axial,AM1,AM2
      WRITE(6,600)K,Axial,AM1,AM2
      IF(K.GT.NM)GO TO 50
      AR(3*K-2)=Axial
      AR(3*K-1)=AM1
      AR(3*K)=AM2
600   FORMAT(3X,I5,1X,3(E10.4,5x))
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces.To do this,identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      ALX=X(JE(K))-X(JS(K))
      ALY=Y(JE(K))-Y(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/AL
      SIN=ALY/AL
      A1=-AR(3*K-2)
      A2=(AR(3*K-1)+AR(3*K))/AL
      A5=-A2
      N1=3*JS(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+A1*C-A2*SIN
      F(N2)=F(N2)+A1*SIN+A2*C
      F(N3)=F(N3)+AR(3*K-1)
      N1=3*JE(K)-2
      N2=N1+1
      N3=N1+2
      F(N1)=F(N1)+AR(3*K-2)*C-A5*SIN
      F(N2)=F(N2)+AR(3*K-2)*SIN+A5*C
      F(N3)=F(N3)+AR(3*K)
      GO TO 40
50    CONTINUE
c     Read initial axial forces  member-end moments due to prestressing. The
c     displacement caused by the prestressing are not determined by this
c     analysis. For each prestressed member, read one line composed of an
c     integer (member number) and three real values. The first real value is
c     the axial force (positive when tensile) the remaining two are the end
c     moments at first and second node ,respectively(positive when clockwise.
c     The last line in this set of data lines must start with an integer
c     greater than NM;the line must be completed by  three real numbers.
c     Generate {FINIT}=initial nodal forces which must exist to equilibrate
c     the given initial member end forces. The displacements due to the
c     initial forces are not of concern of this computer program.
      WRITE(6,450)
450   FORMAT(/,3X,'Member end forces due to initial prestressing')
      WRITE(6,500)
60    CONTINUE
      READ(5,*)K,Axial,AM1,AM2
      WRITE(6,600)K,Axial,AM1,AM2
      IF(K.GT.NM)GO TO 70
      AINIT(3*K-2)=Axial
      AINIT(3*K-1)=AM1
      AINIT(3*K)=AM2
      A1=-Axial
      ALX=X(JE(K))-X(JS(K))
      ALY=Y(JE(K))-Y(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/AL
      SIN=ALY/AL
      A2=(AM1+AM2)/AL
      A5=-A2
      FINIT(3*JS(K)-2)=FINIT(3*JS(K)-2)+A1*C-A2*SIN
      FINIT(3*JS(K)-1)=FINIT(3*JS(K)-1)+A1*SIN+A2*C
      FINIT(3*JS(K))=FINIT(3*JS(K))+AINIT(3*K-1)
      FINIT(3*JE(K)-2)=FINIT(3*JE(K)-2)+AINIT(3*K-2)*C-A5*SIN
      FINIT(3*JE(K)-1)=FINIT(3*JE(K)-1)+AINIT(3*K-2)*SIN+A5*C
      FINIT(3*JE(K))=FINIT(3*JE(K))+AINIT(3*K)
      GO TO 60
70    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,'Initial nodal forces necessary for equilibrium of',/
     + ,3X,'the initial prestressing forces given above')
      WRITE(6,200)
      DO 75 I=1,NJ
75    WRITE(6,300)I,FINIT(3*I-2),FINIT(3*I-1),FINIT(3*I)
      WRITE(6,149)
149   FORMAT(3X,'------------------------------------------',
     +'---------------------------------------')
      RETURN
      END

      SUBROUTINE UPDATEF(NJ3,F,FUPDAT,STEPS,NSTG,NSTEP,PD,PDUPDAT,SUMF,
     + NS,ISUP)
c     Calculation of the nodal forces after applying new load stage.
c     Also, determine the prescribed displacement after introducing
c     afraction in the new load stage.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION F(NJ3),FUPDAT(NJ3),STEPS(NSTG),PD(NSJ,3),PDUPDAT(NSJ,3)
      DIMENSION NS(NSJ),ISUP(NSJ,3)
      COOF=0.
      DO 5 K=1,NSTEP
5     COOF=COOF+STEPS(K)
      DO 10 I=1,NJ3
10    FUPDAT(I)=COOF*F(I)
      DO 15 I=1,NSJ
      DO 15 J=1,3
15    PDUPDAT(I,J)=STEPS(NSTEP)*PD(I,J)
      IF(NSTEP.EQ.1)GO TO 16
      COOF=COOF-STEPS(NSTEP)
16    CONTINUE
c     Calculate sum of squares of nodal forces before the current load stage.
c     Exclude forces at coordinates with prescribed displacement.
      SUMF=0.
      DO 20 I=1,NJ
      DO 17 K=1,NSJ
17    IF(I.EQ.NS(K))GO TO 18
      SUMF=SUMF+(COOF*F(3*I-2))**2
      SUMF=SUMF+(COOF*F(3*I-1))**2
      SUMF=SUMF+(COOF*F(3*I))**2
      GO TO 19
18    CONTINUE
      IF(ISUP(K,1).EQ.1)SUMF=SUMF+(COOF*F(3*I-2))**2
      IF(ISUP(K,2).EQ.1)SUMF=SUMF+(COOF*F(3*I-1))**2
      IF(ISUP(K,3).EQ.1)SUMF=SUMF+(COOF*F(3*I))**2
19    CONTINUE
20    CONTINUE
      SUMF=DSQRT(SUMF)
      RETURN
      END

      SUBROUTINE OOBFS(JS,JE,XUPDAT,YUPDAT,FUPDAT,AUPDAT,NJ3,OOBF,
     +	SUMOOBF, NS,ISUP,SUMR,R,FINIT,RSAVE,NSTEP,STEPS,NSTG,AR,X,Y)
c     Calculation of out of balance forces at the start of an iteration cycle.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION OOBF(NJ3),FUPDAT(NJ3),AUPDAT(3*NM),R(NSJ,3),FINIT(3*NJ)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),JS(NM),JE(NM),NS(NSJ),ISUP(NSJ,3)
      DIMENSION RSAVE(NSJ,3),STEPS(NSTG),AR(3*NM),X(NJ),Y(NJ)
      COOF=0.
      DO 2  I=1,NSTEP
2     COOF=COOF+STEPS(I)
      DO 5 I=1,NJ3
5     OOBF(I)=FUPDAT(I)
      DO 10 L=1,NM
      ALX=XUPDAT(JE(L))-XUPDAT(JS(L))
      ALY=YUPDAT(JE(L))-YUPDAT(JS(L))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/AL
      SIN=ALY/AL

c     Each member contributes a set of self equilibrating forces to the
c     vector {OOBF}.To ensure this, the transverse end forces are considered
c     equal to (+ or -) the sum of the end moments divided by the updated
c     length of member.
      A1=-AUPDAT(3*L-2)
      A2=( AUPDAT(3*(L-1)+2)+AUPDAT(3*(L-1)+3) )/AL
      A5=-A2

c     Add the components -in the global directions- of the updated member end
c     forces to the vector of out-of-balance forces.

      N1=3*JS(L)-2
      N2=N1+1
      N3=N1+2
      OOBF(N1)=OOBF(N1)+A1*C-A2*SIN
      OOBF(N2)=OOBF(N2)+A1*SIN+A2*C
      OOBF(N3)=OOBF(N3)+AUPDAT(3*L-1)
      N1=3*JE(L)-2
      N2=N1+1
      N3=N1+2
      OOBF(N1)=OOBF(N1)+AUPDAT(3*L-2)*C-A5*SIN
      OOBF(N2)=OOBF(N2)+AUPDAT(3*L-2)*SIN+A5*C
      OOBF(N3)=OOBF(N3)+AUPDAT(3*L)

c     Nodal forces due to {Ar} are included in {FUPDAT} and in {AUPDAT} which
c     are used above to generate {OOBF}. Deduct below the nodal forces due to
c     {Ar}; for this purpose refer to the initial geometry.
      ALXIN=X(JE(L))-X(JS(L))
      ALYIN=Y(JE(L))-Y(JS(L))
      ALIN=DSQRT(ALXIN*ALXIN+ALYIN*ALYIN)
      CIN=ALXIN/ALIN
      SININ=ALYIN/ALIN

      A4=COOF*AR(3*L-2)
      A1=-A4
      A3=COOF*AR(3*L-1)
      A6=COOF*AR(3*L)
      A2=(A3+A6)/ALIN
      A5=-A2
      OOBF(3*JS(L)-2)=OOBF(3*JS(L)-2)-(A1*CIN-A2*SININ)
      OOBF(3*JS(L)-1)=OOBF(3*JS(L)-1)-(A1*SININ+A2*CIN)
      OOBF(3*JS(L))=OOBF(3*JS(L))-A3
      OOBF(3*JE(L)-2)=OOBF(3*JE(L)-2)-(A4*CIN-A5*SININ)
      OOBF(3*JE(L)-1)=OOBF(3*JE(L)-1)-(A4*SININ+A5*CIN)
      OOBF(3*JE(L))=OOBF(3*JE(L))-A6
10    CONTINUE
      DO 151 I=1,NJ3
151   OOBF(I)=OOBF(I)-FINIT(I)
c     The reactions are calculated as the forces balancing the member end
c     forces (see below). Therefore, the out-of-balance force at a coord-
c     inate with prescribed displacement is set to zero.
      DO 30 I=1,NSJ
      IF(ISUP(I,1).EQ.0)OOBF(3*NS(I)-2)=0.
      IF(ISUP(I,2).EQ.0)OOBF(3*NS(I)-1)=0.
30    IF(ISUP(I,3).EQ.0)OOBF(3*NS(I))=0.
c     Sum the square of out-of-balance forces for assesment of error ratio.
      SUMOOBF=0.
      DO 20 I=1,NJ3
20    SUMOOBF=SUMOOBF+ OOBF(I)*OOBF(I)
      SUMOOBF=SQRT(SUMOOBF)
c     Calculate the updated reactions from the updated member end forces.
      DO 1 I=1,NSJ
      DO 1 J=1,3
1     R(I,J)=0.
      DO 16 L=1,NSJ
      DO 15 K=1,NM
      IF((NS(L).NE.JS(K)).AND.(NS(L).NE.JE(K)))GO TO 15
      ALX=XUPDAT(JE(K))-XUPDAT(JS(K))
      ALY=YUPDAT(JE(K))-YUPDAT(JS(K))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/AL
      SIN=ALY/AL
      A4=AUPDAT(3*K-2)
      A1=-A4
      A3=AUPDAT(3*K-1)
      A6=AUPDAT(3*K)
      A2=(A3+A6)/AL
      A5=-A2
      IF(JS(K).EQ.NS(L))R(L,1)=R(L,1)+(A1*C-A2*SIN)
      IF(JS(K).EQ.NS(L))R(L,2)=R(L,2)+(A1*SIN+A2*C)
      IF(JS(K).EQ.NS(L))R(L,3)=R(L,3)+A3
      IF(JE(K).EQ.NS(L))R(L,1)=R(L,1)+(A4*C-A5*SIN)
      IF(JE(K).EQ.NS(L))R(L,2)=R(L,2)+(A4*SIN+A5*C)
      IF(JE(K).EQ.NS(L))R(L,3)=R(L,3)+A6
15    CONTINUE
16    CONTINUE
c     Determine sum of squares of the reaction components.
      SUMR=0.
      DO 21 I=1,NSJ
      DO 21 J=1,3
      IF(ISUP(I,J).EQ.0)SUMR=SUMR+R(I,J)*R(I,J)
21    CONTINUE
      SUMR=SQRT(SUMR)
c     The reactions calculated from member end forces do not include the effect
c     of external forces applied along a reaction component. Add the missing
c     values.
      DO 22 I=1,NSJ
      DO 22 J=1,3
      IF(ISUP(I,J).EQ.0)R(I,J)=R(I,J)+COOF*RSAVE(I,J)
22    CONTINUE
      RETURN
      END

      SUBROUTINE ENDRS(ALJINIT,AIJ,E,P,SS,T)
c     Member end rotational stiffness and carryover moment accounting for
c     the effect of axial force (Eqs. 14-17,18,21 and 22 of Ghali and Neville,
c     4th. ed. .
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      U=ALJINIT*DSQRT(ABS(P)/(E*AIJ))
      IF(U.LT.0.07)SS=(4.*E*AIJ)/ALJINIT
      IF(U.LT.0.07)T=(2.*E*AIJ)/ALJINIT
      IF(U.LT.0.07)GO TO 10
      IF(P.GT.0.0)GO TO 20
      SS=(U*(DSIN(U)-U*DCOS(U)))/(2.-2.*DCOS(U)-U*DSIN(U))
      SS=(SS*E*AIJ)/ALJINIT
      T=(U*(U-DSIN(U)))/(2.-2.*DCOS(U)-U*DSIN(U))
      T=(T*E*AIJ)/ALJINIT
      GO TO 10
20    CONTINUE
      IF(U.GT.100.)SS=U*(U-1.)/(U-2.)
      IF(U.GT.100.)GO TO 30
      SH=0.5*(DEXP(U)-DEXP(-U))
      CH=0.5*(DEXP(U)+DEXP(-U))
      SS=(U*(U*CH-SH))/(2.-2.*CH+U*SH)
30    SS=(SS*E*AIJ)/ALJINIT
      IF(U.GT.100.)T=U/(U-2.)
      IF(U.GT.100.)GO TO 40
      T=(U*(SH-U))/(2.-2.*CH+U*SH)
40    T=(T*E*AIJ)/ALJINIT
10    CONTINUE
      RETURN
      END

      SUBROUTINE SUPTPF(S,OOBF,NS,ISUP,PDUPDAT,NJ3,NB,ITER,IREFINE)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 22-39 of Ghali and Neville.Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION S(NJ3,NB),OOBF(NJ3),NS(NSJ),ISUP(NSJ,3)
      DIMENSION PDUPDAT(NSJ,3)
      DO 50 I=1,NSJ
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      N=N1+K
      SKM=S(N,1)
      S(N,1)=S(N,1)*1.0E6
      IF((ITER.GT.1).OR.(IREFINE.EQ.1))GO TO 30
      OOBF(N)=-SKM*1.0E6*PDUPDAT(I,K)
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTPF(DUPDAT,NS,R,AUPDAT,NJ3,ITER,ERROR,IREFINE,
     + X,Y,JS,JE)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION DUPDAT(NJ3),NS(NSJ),AUPDAT(3*NM)
      DIMENSION R(NSJ,3),X(NJ),Y(NJ),JS(NM),JE(NM)
      WRITE(6,1000)
1000  FORMAT(3X,'------------------------------------------------------
     *-------------------------------------------------')
      IF(IREFINE.EQ.0)WRITE(6,1100)
1100  FORMAT(3X,'Analysis results')
      IF(IREFINE.EQ.1)WRITE(6,1101)
1101  FORMAT(3X,'Results after refinement')
      WRITE(6,200)
200   FORMAT(/,3X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'v',13x,'theta')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,DUPDAT(N1),DUPDAT(N2),DUPDAT(N3)
400   FORMAT(3X,I5,3(1X,E16.8))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,3X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fx',14X,'Fy',14X,'Mz')
      DO 30 L=1,NSJ
      WRITE(6,400)NS(L),R(L,1),R(L,2),R(L,3)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,'Member end forces')
      WRITE(6,800)
800   FORMAT(3X,'Member',7X,'F1',10X,'F2',10x,'F3',10x,'F4',10X,'F5',
     *10X,'F6')
      DO 50 JM=1,NM
      ALX=X(JE(JM))+DUPDAT(3*JE(JM)-2)-(X(JS(JM))+DUPDAT(3*JS(JM)-2))
      ALY=Y(JE(JM))+DUPDAT(3*JE(JM)-1)-(Y(JS(JM))+DUPDAT(3*JS(JM)-1))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      A4=AUPDAT(3*JM-2)
      A1=-A4
      A3=AUPDAT(3*JM-1)
      A6=AUPDAT(3*JM)
      A2=(A3+A6)/AL
      A5=-A2
      WRITE(6,900)JM,A1,A2,A3,A4,A5,A6
900   FORMAT(3X,I5,6(1X,E11.5))
50    CONTINUE
      IF(IREFINE.EQ.0)WRITE(6,1250)(ITER-1),ERROR
1250  FORMAT(/,3X,'Number of iterations performed in this stage=',I2,/,
     +3X,'Error=',E12.5)
      IF(IREFINE.EQ.1)WRITE(6,1251)ERROR
1251  FORMAT(/,3X,'Error=',E12.5)
      WRITE(6,1200)
1200  FORMAT(3X,'-------------------------------------------------------
     *------------------------------------------------')
60    CONTINUE
      RETURN
      END

      SUBROUTINE UPDATED(DUPDAT,D,NJ3)
c     Update the reaction vector {RSUM}.
c     Add the increment in displacements to the existing values stored in
c     {DSUM}.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION D(NJ3),DUPDAT(NJ3)
      DO 10 I=1,NJ3
10    DUPDAT(I)=DUPDAT(I)+D(I)
      RETURN
      END

      SUBROUTINE UPDATEA(D,XUPDAT,YUPDAT,JS,JE,A,AI,AUPDAT,NJ3,X,Y,
     + IREFINE,PVALUE)
c     Update member end forces by adding the effect of the displacement
c     increment {D}.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION D(NJ3),AUPDAT(3*NM),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION XUPDAT(NJ),YUPDAT(NJ),X(NJ),Y(NJ)
      DIMENSION PVALUE(NM)
      DO 10 K=1,NM
      ALX=XUPDAT(JE(K))-XUPDAT(JS(K))
      ALY=YUPDAT(JE(K))-YUPDAT(JS(K))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      DELX=C*(D(3*JE(K)-2)-D(3*JS(K)-2))+SIN*(D(3*JE(K)-1)-D(3*JS(K)-1))
      DELY=C*(D(3*JE(K)-1)-D(3*JS(K)-1))-SIN*(D(3*JE(K)-2)-D(3*JS(K)-2))
      ALPHA=DATAN2(DELY,(ALJ+DELX))
      THETA1=D(3*JS(K))-ALPHA
      THETA2=D(3*JE(K))-ALPHA
      AIJ=AI(K)
      ALJINIT=DSQRT( (X(JE(K))-X(JS(K))) *(X(JE(K))-X(JS(K)))
     +		   +(Y(JE(K))-Y(JS(K))) *(Y(JE(K))-Y(JS(K)))  )
      PP=AUPDAT(3*K-2)
      IF(IREFINE.EQ.1)PP=PVALUE(K)
      IF(IREFINE.EQ.1)CALL ENDRS(ALJINIT,AIJ,E,PP,SS,T)
      IF(IREFINE.EQ.0)SS=4.*E*AIJ/ALJINIT
      IF(IREFINE.EQ.0)T=SS/2.
      DELTAL=DSQRT((ALJ+DELX)*(ALJ+DELX)+DELY*DELY)-ALJ
      AUPDAT(3*K-2)=AUPDAT(3*K-2)+DELTAL*E*A(K)/ALJINIT
      AUPDAT(3*K-1)=AUPDAT(3*K-1)+SS*THETA1+T*THETA2
      AUPDAT(3*K)  =AUPDAT(3*K)+SS*THETA2+T*THETA1

c     Note the transverse member end forces are updated from the end moments
c     and determined in the perpidicular direction to the member after the
c     x and y coordinates are updated. This is done in subroutine OOBFS.
10    CONTINUE
      RETURN
      END

      SUBROUTINE UPDATEX_Y(XUPDAT,YUPDAT,D,NJ3)
c     Update the coordinates of nodes.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION D(NJ3),XUPDAT(NJ),YUPDAT(NJ)
      DO 10 I=1,NJ
      XUPDAT(I)=XUPDAT(I)+D(3*I-2)
10    YUPDAT(I)=YUPDAT(I)+D(3*I-1)
      RETURN
      END

      SUBROUTINE VERIFYA(DUPDAT,X,Y,JS,JE,A,AI,AUPDAT,NJ3,
     + PVALUE)
c     Calculate member end forces from updated displacements.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION DUPDAT(NJ3),AUPDAT(3*NM),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION X(NJ),Y(NJ)
      DIMENSION PVALUE(NM)
      DO 10 K=1,NM
      ALX=X(JE(K))-X(JS(K))
      ALY=Y(JE(K))-Y(JS(K))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      DELX=C*(DUPDAT(3*JE(K)-2)-DUPDAT(3*JS(K)-2))+
     +	   SIN*(DUPDAT(3*JE(K)-1)-DUPDAT(3*JS(K)-1))
      DELY=C*(DUPDAT(3*JE(K)-1)-DUPDAT(3*JS(K)-1))-
     +	   SIN*(DUPDAT(3*JE(K)-2)-DUPDAT(3*JS(K)-2))
      ALPHA=DATAN2(DELY,(ALJ+DELX))
      THETA1=DUPDAT(3*JS(K))-ALPHA
      THETA2=DUPDAT(3*JE(K))-ALPHA
      AIJ=AI(K)
      PP=PVALUE(K)
      CALL ENDRS(ALJ,AIJ,E,PP,SS,T)
      DELTAL=DSQRT((ALJ+DELX)*(ALJ+DELX)+DELY*DELY)-ALJ
      AUPDAT(3*K-2)=AUPDAT(3*K-2)+DELTAL*E*A(K)/ALJ
      AUPDAT(3*K-1)=AUPDAT(3*K-1)+SS*THETA1+T*THETA2
      AUPDAT(3*K)=AUPDAT(3*K)+T*THETA1+SS*THETA2
10    CONTINUE
      RETURN
      END

      SUBROUTINE PVALUES(AUPDAT,PVALUE)
c     Store the values of the axial forces in the members in {PVALUE},
c     to be used as constants in refinement  calculations.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E
      DIMENSION AUPDAT(3*NM),PVALUE(NM)
      DO 10 I=1,NM
10    PVALUE(I)=AUPDAT(3*I-2)
      RETURN
      END

c		 Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c		   Based on Ghali,A. and Neville,A. M.,
c      Structural Analysis :A Unified Classical and Matrix Approach
c	Chapman and Hall,London and New York,3 ed.,1990,Chapter 24
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 24-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------

c			  Program Listing
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 25-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S(NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T


c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98

50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 24-49 to 24-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 24-48), determine [g]
c     and store in [D].

      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)

c     By Eq. 24-46 of Ghali and Neville and backward substitution(Eq.24-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,3X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,3X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,3x,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,3X,
     +'on the diagonal in solution of equations on row:',I4)

      STOP
99    CONTINUE
      RETURN
      END
