c			 Computer Program PLANEF
c                       Analysis of Plane Frames         
c                           January 2016
c			     by A. Ghali
c                Emeritus Professor of Civil Engineering
c                        The University of Calgary
c                         Calgary, Canada T2N 1N4

c  Based on Ghali,A., Neville,A. M., Structural Analysis :A Unified Classical
c  and Matrix Approach, CRC Press, Spon Press, 7th ed., 2016.
c------------------------------------------------------------------------------
c				 Scope
c     PLANEF analyses frames with straight prismatic members ,for which the
c  centroidal axes are in the xy plane. Axial,bending and shear deformations
c  are considered.The analysis gives the nodal displacements, the reactions
c  and the member end forces.                                              

c     The loading can be any, or cmbination,of the following types:
c      (a) Nodal forces.Two concentrated forces and a couple:Fx,Fy and Mz.
c      (b) General load or temperature variation in a member; in which case
c          the six member fixed-end forces ,{Ar} must be included in the
c          input .
c
c			      Sign Convention
c     Positive directions of nodal displacements u,v and theta and  member
c  end forces are defined in Figs. 21-1 and 21-2 of Ghali et al. The
c  reaction components are determined in the directions of the global axes.
c
c			  Input Data Instructions
c  The data, in free format, must be in a file named PLANEF.IN. Any editor
c  may be used (e.g. Edit of DOS or Notepad of Microsoft). The data is
c  composed of single lines or  sets of lines as follows:
c  - Single line of title ,not exceeding 72 characters.
c  - Single line of 4 integers: number of nodes,number of members,number of
c    joints with prescribed displacement(s) and number of load cases.
c  - Single line of two real values for E and Poisson's ratio.
c  - A set of lines; their number equals number of nodes. Each line is 
c    composed of a node number followed by its x and y coordinates.
c  - A set of lines their number equals number of members. Each line starts
c    with 3 integers: a member number, its starting node and its end node,
c      followed by three real values : a,I and ar.
c  - A set of lines their number equals number of joints with prescribed 
c      displacements. Each line starts with a node number and three restraint
c      indicators (0 or 1), followed by  three	real values of	prescribed
c      displacements. The restraint indicators 1 and 0 mean free and pres-
c      cribed displacement component, respectively. When a displacement 
c      component is free, a dummy value must be filled in for the prescribed 
c      displacement (any real number).
c  - A set of any number of lines for loading of type(a).Each line is comp-
c      osed of two integers, a load case number and a node number, followed by
c      three real values of nodal forces Fx,Fy and Mz. The last line of the 
c      set must start by an integer greater than the number of load cases 
c      line must not be omitted, even when there is no loading of type (a).
c  - A set of any number of lines for loading of type(b). Each line is comp-
c      osed of two integers, a load case number and a member number, followed by
c      six real values of member end forces. The last line of this set must
c      start with an integer greater than the number of load cases followed
c      by a dummy integer and six dummy real values. The last line must not 
c      be omitted even when there is no loading of type (b).

c    Note that the prescribed displacement data apply to all load cases. If
c  this is not what is required the analysis for a support movement is to be
c  done separately. This restriction can be avoided by entering zero precribed
c  displacements and giving the member end forces when the prescribed displac-
c  ements occur while the other displacements are zero(See Example 21-2 of
c  Ghali et al).

c    Examle input data:

c  Plane frame of Example 23-2,Fig. 23-5 of Ghali et al.
c   4	 3    2    2			NJ,NM,NSJ,NLC
c   30.0e3	   0.0			Elasticity modulus, Poisson's ratio
c   1	      0.0	300.0		Node no., x and y coordinates
c   2	      0.0	  0.0
c   3	    300.0	150.0
c   4	    300.0	300.0
c   1	 1    2   30.0	3000.0	1.0e6	Member no.,JS,JE,cross-sec. area,I,ar
c   2	 2    3   40.0	5000.0	1.0e6
c   3	 3    4   30.0	3000.0	1.0e6
c   1	 0   0	 0   0.0   0.0	  0.0	Node no.,restr. indics.,prsc. displs.
c   4	 0   0	 1   0.0   0.0	  0.0
c   1	 2    4.0    0.0    0.0 	Ld. case no.,node,Fx,Fy and Mz
c   1	 3    2.0    0.0    0.0
c  10	 0    0.0    0.0    0.0 	Dummy,end of data of loaded nodes
c   1  2  -13.4164 -26.8328 -1500.0 -13.4164 -26.8328 1500.0  Case,member,{Ar}
c   2  1  -1500.0    8.0     1200.0 1500.0   -8.0      1200.0
c  10  0  0.0   0.0   0.0   0.0   0.0   0.0  Dummy,end of data of member loads

c    When it is desired to ignore axial or shear deformatios(e.g. to compare
c  the results with hand solution),enter large values for a or ar. Note that
c  using excessively large number for a may produce numerical difficulty in 
c  solution of the equilibrium equation, giving wrong displacements. In the 
c  above example, the intension is to ignore shear deformation.
c    The letter c at the start of each data line  must, of course, be omitted.
c  To run the program, type PLANEF and press Enter. The program will execute
c  the analysis and give the input and the results in a file named PLANEF.OUT .

c			     Limits on Data
c    Vectors {B} and {NI} are used to store all the real and the integer arrays
c  used by the program. The maximum dimensions set are: MAXB=20000 and MAXNI=
c  2000 . This would normally be sufficient for a frame approximately 120 nodes,
c  120 members and 8 load cases, depending upon the band width. The maximum 
c  dimensions may be changed as required on line 2 of program(see listing 
c  below). After the change, PLANEF.FOR must be compiled using a fortran
c  compiler(e.g. Microsoft Fortran Optimizing compiler).

c    The dimension of the vectors used for a given problem may be calculated
c  by:

c  Dimension of {B}=NJ*(2+6*NLC+3*NB)+3*NM+3*NSJ*(NLC+2)+6*(NRAR+1)

c  Dimension of{NI}=2*NM+4*NSJ+2*(NRAR+1)
c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c                                                                   rows cols.
c  {A}    cross-section areas                                       NM
c  {AI}   second moments of areas of cross-sections                 NM
c  {ARD}  effective shear cross-section areas                       NM
c  [AR]   member end forces with nodal displacements restrained     6    NRAR
c  [D]    nodal displacements                                       3*NJ  NLC
c  E	  modulus of elasticity
c  [ES]   element (member) stiffness in local coordinates           6      6
c  [EST]  element stiffness matrix in directions parallel to        6      6
c	  global axes
c  [F]    nodal restraining forces                                 3*NJ   NLC
c  [ISUP] joint restraint indicators: 1 and 0, indicating free      NSJ    3
c	  and prescribed displacement, respectively
c  {JE}   end nodes of individual members                           NM
c  {JS}   starting node of individual members                       NM
c  [LR]   list of load case and member number for the members        2   NRAR
c	   which fixed-end forces  are given as data
c  NB     band width ; see Sec. 21-3 of Ghali et al.
c  NJ	  number of joints (nodes)
c  NLC	  number of load cases
c  NM	  number of members
c  NRAR   number of rows in [AR]; this is equal the number of 
c         members for which fixed-end forces are given as data
c  {NS}   list of node numbers where supports are provided or       NSJ
c         displacements are prescribed
c  NSJ    number of supprted joints (nodes with prescribed dis-
c         placements)
c  [PD]   prescribed displacement values                            NSJ   3
c  PR	  Poisson's ratio
c  [R]    forces at the supprted nodes including the reactions     3*NSJ NLC
c  [S]    stiffness matrix                                         3*NJ   NB
c  {SK}   diagonal elements of [S] corresponding to  coordinates   3*NSJ
c	  at the supportd nodes
c  {X}    x coordinate of nodes                                      NJ
c  {Y}	  y coordinates of nodes
c------------------------------------------------------------------------------
c		 Listing of program PLANEF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='PLANEF.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='PLANEF.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(1X,A76)
      READ(5,*)NJ,NM,NSJ,NLC
      WRITE(6,300)NJ
300   FORMAT(1X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(1X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(1X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NLC
600   FORMAT(1X,'Number of load cases=',I5)
      NJ3=3*NJ
      NS3=3*NSJ
c     Elements of {B} at which real arrays start are:
c     {A}:N1  {AI}:N2  {ARD}:N3  [D]:N4  [F]:N5  [PD]:N6  [R]:N7  {SK}:N8
c     {X}:N9  {Y}:N10  [S]:N11	[AR]:N12
      N1=1
      N2=N1+NM
      N3=N2+NM
      N4=N3+NM
      N5=N4+NJ3*NLC
      N6=N5+NJ3*NLC
      N7=N6+NS3
      N8=N7+NS3*NLC
      N9=N8+NS3
      N10=N9+NJ
      N11=N10+NJ
c     Elements of {NI} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  [LR]:K5
      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      K5=K4+NS3
c     CALL DATAPF(X,Y,JS,JE,A,AI,ARD,NS,ISUP,PD,NB)
      CALL DATAPF(B(N9),B(N10),NI(K1),NI(K2),B(N1),B(N2),B(N3),NI(K3),
     *NI(K4),B(N6),NB)
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N12=N11+NJ3*NB
c     Determine maximum number of rows of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     rows of [R] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N12)/6
      NLR=(MAXNI-K5)/2
c     CALL STIFPF(JS,JE,A,AI,ARD,X,Y,S,NJ3,NB)
      CALL STIFPF(NI(K1),NI(K2),B(N1),B(N2),B(N3),B(N9),B(N10),B(N11),
     *NJ3,NB)
c     CALL LOADPF(JS,JE,X,Y,F,LR,AR,NJ3,NLR,NAR,NLC)
      CALL LOADPF(NI(K1),NI(K2),B(N9),B(N10),B(N5),NI(K5),B(N12),NJ3,
     *NLR,NAR,NLC)
c     CALL SUPTPF(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
      CALL SUPTPF(B(N11),B(N5),NI(K3),NI(K4),B(N6),B(N7),B(N8),NJ3,NS3,
     *NB,NLC)
c     CALL EQSOLN(S,F,D,NJ3,NB,NLC)
      CALL EQSOLN(B(N11),B(N5),B(N4),NJ3,NB,NLC)
c     CALL RSLTPF(D,NS,ISUP,PD,R,SK,X,Y,JS,JE,A,AI,ARD,LR,AR,NLR,NAR,
c    *NJ3,NS3,NLC)
      CALL RSLTPF(B(N4),NI(K3),NI(K4),B(N6),B(N7),B(N8),B(N9),B(N10),
     *NI(K1),NI(K2),B(N1),B(N2),B(N3),NI(K5),B(N12),NLR,NAR,NJ3,NS3,
     *NLC)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATAPF(X,Y,JS,JE,A,AI,ARD,NS,ISUP,PD,NB)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION X(NJ),Y(NJ),JS(NM),JE(NM),A(NM),AI(NM),ARD(NM)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      READ(5,*)E,PR
      WRITE(6,200)E,PR
200   FORMAT(1X,'Elasticity modulus=',E13.5,2X,';Poissons ratio=',F6.4)
      WRITE(6,300)
300   FORMAT(/,1X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',5x,'x',9X,'y')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J)
10    WRITE(6,500)J,X(J),Y(J)
500   FORMAT(1X,I5,2F10.3)
      WRITE(6,600)
600   FORMAT(/,1X,'Element information')
      WRITE(6,700)
700   FORMAT(1X,'Element',2X,'1st node',3X,'2nd node',6x,'a',13x,'I',
     *13x,'ar')
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),A(J),AI(J),ARD(J)
c     Member no.,starting node,end node,cross-section area,2nd moment
c     of area and area reduced.
20    WRITE(6,800)J,JS(J),JE(J),A(J),AI(J),ARD(J)
800   FORMAT(1X,I5,4X,I5,5X,I5,3X,3(E13.5,1X))
      WRITE(6,900)
900   FORMAT(/,1X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',5X,'Restraint indicators',7X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or o,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'v',3X,'theta',8x,'u',13x,'v',12x,'theta')
1200  FORMAT(1X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 23-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFPF(JS,JE,A,AI,ARD,X,Y,S,NJ3,NB)
c     Stiffness matrix for plane frame.See Sec. 23-8,Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),A(NM),AI(NM),ARD(NM),X(NJ),Y(NJ)
      DIMENSION S(NJ3,NB)
      DIMENSION ES(6,6),EST(6,6),IM(6)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      AJ=A(JM)
      AIJ=AI(JM)
      ARDJ=ARD(JM)
c     Member stiffness matrix in local coordinates,considering shear
c     deformation.
      CALL ESTFPF(ALJ,AJ,AIJ,ARDJ,E,PR,ES)
c     Transform element stiffness,[ES] into [EST] in global directions;
c     see Eq. 23-27 of Ghali and Nevlle.
      DO 20 I=1,6
      DO 20 J=1,6
20    EST(I,J)=0.0
      EST(1,1)=ES(1,1)*C*C+ES(2,2)*SIN*SIN
      EST(4,4)=EST(1,1)
      EST(4,1)=-EST(1,1)
      EST(3,3)=ES(3,3)
      EST(6,6)=EST(3,3)
      EST(2,2)=ES(1,1)*SIN*SIN+ES(2,2)*C*C
      EST(5,5)=EST(2,2)
      EST(5,2)=-EST(2,2)
      EST(6,3)=ES(6,3)
      EST(2,1)=(ES(1,1)-ES(2,2))*C*SIN
      EST(5,1)=-EST(2,1)
      EST(5,4)=EST(2,1)
      EST(3,1)=-ES(3,2)*SIN
      EST(4,3)=-EST(3,1)
      EST(6,1)=EST(3,1)
      EST(6,4)=-EST(3,1)
      EST(3,2)=ES(3,2)*C
      EST(5,3)=-EST(3,2)
      EST(4,2)=-EST(2,1)
      EST(6,2)=EST(3,2)
      EST(6,5)=-EST(3,2)
      DO 30 I=1,6
      DO 30 J=I+1,6
30    EST(I,J)=EST(J,I)
c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE ESTFPF(ALJ,AJ,AIJ,ARDJ,E,PR,ES)
c     Element stiffness matrix in local coordinates;member of plane
c     frame;shear deformation considered(Eq. 16-5,Ghali et al.
c     ALJ=member length;AJ=a;AIJ=I;ARDJ=ar;PR=Poisson's ratio.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ES(6,6)
      ALPHA=24.0*(1.0+PR)*AIJ/(ALJ*ALJ*ARDJ)
      DO 10 I=1,6
      DO 10 J=1,6
10    ES(I,J)=0.0
      ES(1,1)=E*AJ/ALJ
      ES(4,1)=-ES(1,1)
      ES(4,4)=ES(1,1)
      ES(2,2)=12.0*E*AIJ/((1.0+ALPHA)*ALJ*ALJ*ALJ)
      ES(5,2)=-ES(2,2)
      ES(5,5)=ES(2,2)
      ES(3,3)=(4.0+ALPHA)*E*AIJ/((1.0+ALPHA)*ALJ)
      ES(6,6)=ES(3,3)
      ES(3,2)=ES(2,2)*ALJ/2.0
      ES(6,2)=ES(3,2)
      ES(5,3)=-ES(6,2)
      ES(6,3)=(2.0-ALPHA)*E*AIJ/((1.0+ALPHA)*ALJ)
      ES(6,5)=-ES(3,2)
      DO 20 I=1,6
      DO 20 J=I+1,6
20    ES(I,J)=ES(J,I)
      RETURN
      END

      SUBROUTINE LOADPF(JS,JE,X,Y,F,LR,AR,NJ3,NLR,NAR,NLC)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),X(NJ),Y(NJ),F(NJ3,NLC),LR(2,NLR)
      DIMENSION AR(6,NAR)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of load cases;
c     the line must be completed by another integer and 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,1X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(1X,'Load case ',3X,'Node',7X,'Fx',12X,'Fy',12X,'Mz')
20    CONTINUE
      READ(5,*)LCASE,NODE,FX,FY,AMZ
c     IF(LCASE.GT.NLC)GO TO 30
      WRITE(6,300)LCASE,NODE,FX,FY,AMZ
300   FORMAT(1X,I5,6X,I5,3(1X,E13.5))
      IF(LCASE.GT.NLC)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1,LCASE)=F(N1,LCASE)-FX
      F(N2,LCASE)=F(N2,LCASE)-FY
      F(N3,LCASE)=F(N3,LCASE)-AMZ
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     or due to temperature,etc..Note that the last line in this set of
c     data lines must start with an integer > number of load cases;the
c     line must be completed by another integer and six real numbers.
c     The positive directions of the end forces are shown in Fig. 23-2 of
c     Ghali and Neville
      WRITE(6,400)
400   FORMAT(/,1X,'Member end forces with nodal displacement restrained
     *')
      WRITE(6,500)
500   FORMAT(1X,'Ld. case ',1x,'Member',2X,'Ar1',8X,'Ar2',8X,'Ar3',
     *8X,'Ar4',8X,'Ar5',8X,'Ar6')
      NRAR=0
c     NRAR is number of rows in [AR]; that is equal to the number of
c     lines for loading of type (b). The last line of data is not counted.
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),AR(5,N),
     *AR(6,N)
      WRITE(6,600)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),
     *AR(5,N),AR(6,N)
600   FORMAT(1X,I5,1X,I5,1X,5(E10.4,1x),E10.4)
      IF(LR(1,N).GT.NLC)GO TO 50
      LCASE=LR(1,N)
      MEMBER=LR(2,N)
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces. To do this, identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      N1=3*JS(MEMBER)-2
      N2=N1+1
      N3=N1+2
      ALX=X(JE(MEMBER))-X(JS(MEMBER))
      ALY=Y(JE(MEMBER))-Y(JS(MEMBER))
      AL=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/AL
      SIN=ALY/AL
      F(N1,LCASE)=F(N1,LCASE)+AR(1,N)*C-AR(2,N)*SIN
      F(N2,LCASE)=F(N2,LCASE)+AR(1,N)*SIN+AR(2,N)*C
      F(N3,LCASE)=F(N3,LCASE)+AR(3,N)
      N1=3*JE(MEMBER)-2
      N2=N1+1
      N3=N1+2
      F(N1,LCASE)=F(N1,LCASE)+AR(4,N)*C-AR(5,N)*SIN
      F(N2,LCASE)=F(N2,LCASE)+AR(4,N)*SIN+AR(5,N)*C
      F(N3,LCASE)=F(N3,LCASE)+AR(6,N)
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1
      RETURN
      END

      SUBROUTINE SUPTPF(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
c     Adjust [S] and [F] in accordance with the support conditions. See
c     Eq. 21-39 of Ghali et al. Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION S(NJ3,NB),F(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC),SK(NS3)
      DO 10 I=1,NS3
      SK(I)=0.0
      DO 10 J=1,NLC
10    R(I,J)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eq. 23-
c     38 of Ghali and Neville.
      DO 50 I=1,NSJ
      M1=3*I-3
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)
      S(N,1)=S(N,1)*1.0E6
      DO 20 J=1,NLC
      R(M,J)=F(N,J)
      F(N,J)=-SK(M)*1.0E6*PD(I,K)
20    CONTINUE
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTPF(D,NS,ISUP,PD,R,SK,X,Y,JS,JE,A,AI,ARD,LR,AR,NLR,
     *NAR,NJ3,NS3,NLC)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION D(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC)
      DIMENSION SK(NS3),X(NJ),Y(NJ),JS(NM),JE(NM),A(NM),AI(NM)
      DIMENSION ARD(NM),LR(2,NLR),AR(6,NAR)
      DIMENSION ES(6,6)
      WRITE(6,1000)
1000  FORMAT(1X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(1X,'Analysis results')
      DO 60 K=1,NLC
      WRITE(6,100)K
100   FORMAT(/,1X,'Load case no.',I5)
      WRITE(6,200)
200   FORMAT(/,1X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'v',13x,'theta')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,D(N1,K),D(N2,K),D(N3,K)
400   FORMAT(1X,I5,3(3X,E13.5))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,1X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fx',14X,'Fy',14X,'Mz')
      DO 30 L=1,NSJ
      M1=3*L-3
      N1=3*NS(L)-3
      DO 20 J=1,3
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      R(M,K)=R(M,K)+SK(M)*1.0E6*(PD(L,J)-D(N,K))+SK(M)*D(N,K)
20    CONTINUE
      M1=M1+1
      M2=M1+1
      M3=M1+2
      WRITE(6,400)NS(L),R(M1,K),R(M2,K),R(M3,K)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,1X,'Member end forces')
      WRITE(6,800)
800   FORMAT(1X,'Member',7X,'F1',10X,'F2',10x,'F3',10x,'F4',10X,'F5',
     *10X,'F6')
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY)
      C=ALX/ALJ
      SIN=ALY/ALJ
      AJ=A(JM)
      AIJ=AI(JM)
      ARDJ=ARD(JM)
      CALL ESTFPF(ALJ,AJ,AIJ,ARDJ,E,PR,ES)
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      N4=3*JE(JM)-2
      N5=N4+1
      N6=N4+2
c     Transform displacements at member ends into local coordinates.
      DT1=D(N1,K)*C+D(N2,K)*SIN
      DT2=-D(N1,K)*SIN+D(N2,K)*C
      DT3=D(N3,K)
      DT4=D(N4,K)*C+D(N5,K)*SIN
      DT5=-D(N4,K)*SIN+D(N5,K)*C
      DT6=D(N6,K)
c     For each member,calculate the product of its stiffness matrix by
c     its displacement vector.
      A1=ES(1,1)*DT1+ES(1,4)*DT4
      A2=ES(2,2)*DT2+ES(2,3)*DT3+ES(2,5)*DT5+ES(2,6)*DT6
      A3=ES(3,2)*DT2+ES(3,3)*DT3+ES(3,5)*DT5+ES(3,6)*DT6
      A4=ES(4,1)*DT1+ES(4,4)*DT4
      A5=ES(5,2)*DT2+ES(5,3)*DT3+ES(5,5)*DT5+ES(5,6)*DT6
      A6=ES(6,2)*DT2+ES(6,3)*DT3+ES(6,5)*DT5+ES(6,6)*DT6
c     Add the fixed-end forces included in the input data.
      IF(NRAR.EQ.0)GO TO 45
      DO 40 I=1,NRAR
      IF(LR(1,I).NE.K) GO TO 35
      IF(LR(2,I).NE.JM) GO TO 35
      A1=A1+AR(1,I)
      A2=A2+AR(2,I)
      A3=A3+AR(3,I)
      A4=A4+AR(4,I)
      A5=A5+AR(5,I)
      A6=A6+AR(6,I)
35    CONTINUE
40    CONTINUE
45    CONTINUE
      WRITE(6,900)JM,A1,A2,A3,A4,A5,A6
900   FORMAT(1X,I5,6(1X,E11.5))
50    CONTINUE
      WRITE(6,1200)
1200  FORMAT(1X,'-------------------------------------------------------
     *------------------------------------------------')
60    CONTINUE
      RETURN
      END

  
c		  Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis

c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 21-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 21-11 of Ghali et al.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S[HUGE](NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 21-49 to 21-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 21-48), determine [g]
c     and store in [D].
      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)
c     By Eq. 21-46 of Ghali et al and backward substitution(Eq.21-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,1X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,1X,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
      STOP
99    CONTINUE
      RETURN
      END

