     
c                        Computer Program PLANEG
c                        Analysis of Plane Grids         
c                           January 2016

c			     by A. Ghali
c                 Emeritas Professor of Civil Engineering
c			  The University of Calgary
c			   Calgary, Canada T2N 1N4

c  Based on Ghali,A., Neville,A. M., Structural Analysis :A Unified Classical
c  and Matrix Approach, CRC Press, Spon Press, 7th. ed.,2016,Chapter 23
c------------------------------------------------------------------------------

c				 Scope
c     PLANEG analyses structures with straight prismatic members, for which
c  the centroidal axes are in the xz plane. Deformations due to bending and
c  torsion are considered. The analysis gives the nodal displacements, the
c  reactions and the member end forces.
c     The loading can be any, or cmbination,of the following types:
c      (a) Concentrated force Fy perpendicular to the xz plane and couples Mx
c	   and Mz at the nodes.
c      (b) General load oparallel to the y axis or temperature variation in a
c	   member, in which case six member fixed-end forces,{Ar} must be
c	   included in the input data.
c
c			      Sign Convention
c     Positive directions of nodal displacements w, (theta)x and (theta)z and
c  the  member end forces are defined in  Figs. 23-1 and 23-2 of Ghali and
c  Neville. The reaction components are determined in the  directions of the
c  global axes.

c			  Input Data Instructions
c     The data, in free format, must be in a file named PLANEG.IN. Any editor
c  may be used (e.g. Edlin of DOS or Notepad of Microsoft). The data is
c  composed of single lines or  sets of lines as follows:
c - Single line of title,not exceeding 72 characters.
c - Single line of 4 integers: number of nodes,number of members, number 
c   of joints with prescribed displacement(s) and number of load cases.
c - Single line of two real values: E and Poisson's ratio.
c - A set of lines ;their number equals number of nodes. Each line is
c   compsed of a node number followed by its x and z coordinates.
c - A set of lines their number equals number of members.Each line starts
c   with 3 integers: a member number, its starting node and its end node,
c   followed by two  real values : I and J .
c - A set of lines their number equals number of joints with prescribed
c   displacements. Each line starts with a node number and three restraint
c   indicators (0 or 1), followed by three real values of prescribed
c   displacements. The restraint indicators 1 and 0 mean free and
c   prescribed displacement component, respectively. When a displacement
c   component is free, a dummy value must be filled in for the prescribed
c   displacement (any real number)
c - A set of any number of lines for loading of type(a). Each line is
c   composed of two integers ,a load case number and a node number,
c   followed by three real values of nodal forces Fy,Mx and Mz.The last
c   line of the set must start by an integer greater than the number of
c   load cases followed  by a dummy integer and three dummy real numbers.
c   The last line must not be omitted even when there is no loading of
c   type (a).
c - A set of any number of lines for loading of type(b). Each line is
c   composed of two integers,a load case number and a member number,
c   followed by six real values of member end forces. The last line of
c   this set must start with an integer greater than the number of
c   load cases followed by a dummy integer and six dummy real values.
c   The last line must not be omitted even when there is no loading of
c   type (b).

c Note: The prescribed displacement data apply to all load cases. If
c this is not what is required, the analysis for a support movement is to be
c done separately. To avoid this restriction, enter for the considered support
c integer 0 for a displacement component(s) indicator and precribed 
c displacement-value= 0.0. Then, for each member connected to the node, enter
c values of member-end forces with the  prescribed displacement(s) occuring
c at the support, while other end  of the member fixed.(Example 23-2 of Ghali
c and Neville).


c    Examle input data:

c Plane grid  Example 5-4, Fig. 5-6a  of Ghali and Neville.
c 5    4    4	 1			  NJ,NM,NSJ,NLC
c 1.	    0.0 			  Elasticity modulus, Poisson's ratio
c 1	    0.0       0.66667		  Node no.,x and z coordinates
c 2	    0.66667   1.0
c 3	    0.66667   0.66667
c 4	    0.66667   0.0
c 5	    1.0       0.66667
c 1	1    3	  1.0	 1.0		  Member no.,JS,JE,cross-sec. I and J
c 2	3    2	  1.0	 1.0
c 3	3    5	  1.0	 1.0
c 4	4    3	  1.0	 1.0
c 1	0    0	  0	0.     0.     0.  Node no.,restr. indics.,prsc. displs.
c 2	0    0	  0	0.     0.     0.
c 4	0    0	  0	0.     0.     0.
c 5	0    0	  0	0.     0.     0.
c 10	0    0.     0.	     0. 	  Dummy,end of data of loaded nodes
c 1  1	-.333333  0.0  -37.0370e-3  -.333333  0.0  37.0370e-3  Case,member,{Ar}
c 1  3	-.166667  0.0	-9.25926e-3 -.166667  0.   9.25926e-3
c 10  0   0.0  0.0  0.0  0.0   0.0   0.0   Dummy,end of data of member loads

c    The letter c at the start of each data line  must, of course, be omitted.
c  To run the program, type PLANEG and press Enter. The program will execute
c  the analysis and give the input and the results in a file named PLANEG.OUT.

c			     Limits on Data
c    Vectors {B} and {NI} are used to store all the real and the integer
c  arrays used by the program. The maximum dimensions set are: MAXB=20000 and
c  MAXNI= 2000. This would normally be sufficient for a frame approximately
c  120 nodes,120 members and 8 load cases, depending upon the band width. The
c  maximum dimensions may be changed as required on line 2 of program(see
c  listing below). After the change, the program must be compiled using a
c  FORTRAN compiler(e.g. Microsoft Fortran Optimizing Compiler).

c    The dimensions of the vectors used for a given problem may be calc-
c  ulated by:

c  Dimension of {B}=NJ*(2+6*NLC+3*NB)+2*NM+3*NSJ*(NLC+2)+6*(NRAR+1)

c  Dimension of{NI}=2*NM+4*NSJ+2*(NRAR+1)

c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  {AI}   second moments of areas of cross-sections		     NM
c  {AJ}   torsion constant					     NM
c  [AR]   member end forces with nodal displacements restrained      6	   NRAR
c  [D]	  nodal displacements					    3*NJ   NLC
c  E	  modulus of elasticity
c  [ES]   element (member) stiffness in local coordinates	     6	    6
c  [EST]  element stiffness matrix in directions parallel to	     6	    6
c	  global axes
c  [F]	  nodal restraining forces				    3*NJ   NLC
c  [ISUP] joint restraint indicators : 1 and o ,indicating free      NSJ    3
c	  and prescribed displacement respectively
c  {JE}   end nodes of individual members			     NM
c  {JS}   starting node of individual members			     NM
c  [LR]   list of load case and member number for which fixed-end    2	   NRAR
c	  are given as data
c  NB     band width ; see Sec. 23-3 of Ghali and Neville.
c  NJ	  number of joints (nodes)
c  NLC	  number of load cases
c  NM	  number of members
c  NRAR   number of rows in [AR] ; this is equal the number of mem-
c	  bers for which fixed-end forces are given as data
c  {NS}   list of node numbers where supports are provided or disp-  NSJ
c	  lacements are prescribed
c  NSJ	  number of supprted joints (nodes with prescribed displac-
c	  ements)
c  [PD]   prescribed displacement values			     NSJ   3
c  [R]	  forces at the supprted nodes including the reactions	    3*NSJ NLC
c  [S]	  stiffness matrix					    3*NSJ  NB
c  {SK}   diagonal elements of [S] corresponding to  coordinates    3*NSJ
c	  at the supportd nodes
c  {X}	  x coordinates of nodes				      NJ
c  {Z}	  z coordinates of nodes				      NJ
c------------------------------------------------------------------------------
c		     Listing of program PLANEG
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='PLANEG.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='PLANEG.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(1X,A76)
      READ(5,*)NJ,NM,NSJ,NLC
      WRITE(6,300)NJ
300   FORMAT(1X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(1X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(1X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NLC
600   FORMAT(1X,'Number of load cases=',I5)
      NJ3=3*NJ
      NS3=3*NSJ
c     Elements of {B} at which real arrays start are:
c     {AI}:N1  {AJ}:N2	[D]:N3	[F]:N4	[PD]:N5  [R]:N6  {SK}:N7
c     {X}:N8	{Z}:N9	[S]:N10  [AR]:N11
      N1=1
      N2=N1+NM
      N3=N2+NM
      N4=N3+NJ3*NLC
      N5=N4+NJ3*NLC
      N6=N5+NS3
      N7=N6+NS3*NLC
      N8=N7+NS3
      N9=N8+NJ
      N10=N9+NJ
c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  [LR]:K5
      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      K5=K4+NS3
c     CALL DATAGR(X,Z,JS,JE,AI,AJ,NS,ISUP,PD,NB)
      CALL DATAGR(B(N8),B(N9),NI(K1),NI(K2),B(N1),B(N2),NI(K3),
     *NI(K4),B(N5),NB)
c     Identification of elements of {B} at which real arrays start (continued)
      N11=N10+NJ3*NB
c     Determine maximum number of rows of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     rows of [R] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N11)/6
      NLR=(MAXNI-K5)/2
c     CALL STIFGR(JS,JE,AI,AJ,X,Z,S,NJ3,NB)
      CALL STIFGR(NI(K1),NI(K2),B(N1),B(N2),B(N8),B(N9),B(N10),
     *NJ3,NB)
c     CALL LOADGR(JS,JE,X,Z,F,LR,AR,NJ3,NLR,NAR,NLC)
      CALL LOADPF(NI(K1),NI(K2),B(N8),B(N9),B(N4),NI(K5),B(N11),NJ3,
     *NLR,NAR,NLC)
c     CALL SUPTGR(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
      CALL SUPTGR(B(N10),B(N4),NI(K3),NI(K4),B(N5),B(N6),B(N7),NJ3,NS3,
     *NB,NLC)
c     CALL EQSOLN(S,F,D,NJ3,NB,NLC)
      CALL EQSOLN(B(N10),B(N4),B(N3),NJ3,NB,NLC)
c     CALL RSLTGR(D,NS,ISUP,PD,R,SK,X,Z,JS,JE,AI,AJ,LR,AR,NLR,NAR,
c    *NJ3,NS3,NLC)
      CALL RSLTGR(B(N3),NI(K3),NI(K4),B(N5),B(N6),B(N7),B(N8),B(N9),
     *NI(K1),NI(K2),B(N1),B(N2),NI(K5),B(N11),NLR,NAR,NJ3,NS3,NLC)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATAGR(X,Z,JS,JE,AI,AJ,NS,ISUP,PD,NB)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION X(NJ),Z(NJ),JS(NM),JE(NM),AI(NM),AJ(NM)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      READ(5,*)E,PR
      WRITE(6,200)E,PR
200   FORMAT(1X,'Elasticity modulus=',E13.5,2X,';Poissons ratio=',F6.4)
      WRITE(6,300)
300   FORMAT(/,1X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',6x,'x',9X,'z')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Z(J)
10    WRITE(6,500)J,X(J),Z(J)
500   FORMAT(1X,I5,2F10.3)
      WRITE(6,600)
600   FORMAT(/,1X,'Element information')
      WRITE(6,700)
700   FORMAT(1X,'Element',2X,'1st node',3X,'2nd node',6x,'I',
     *13x,'J')
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),AI(J),AJ(J)
c     Member no.,starting node,end node,2nd moment of cross-section area
c     and tortion constant.
20    WRITE(6,800)J,JS(J),JE(J),AI(J),AJ(J)
800   FORMAT(1X,I5,4X,I5,5X,I5,3X,3(E13.5,1X))
      WRITE(6,900)
900   FORMAT(/,1X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',5X,'Restraint indicators',7X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or o,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'w',1x,'(theta)x',1X,'(theta)z',7x,'w',8x,'(theta)x',
     *8X,'(theta)z')
1200  FORMAT(1X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 23-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFGR(JS,JE,AI,AJ,X,Z,S,NJ3,NB)
c     Stiffness matrix for plane frame.See Sec. 23-9,Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),AI(NM),AJ(NM),X(NJ),Z(NJ)
      DIMENSION S(NJ3,NB)
      DIMENSION EST(6,6),IM(6)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALZ*ALZ)
      C=ALX/ALJ
      SIN=ALZ/ALJ
      AIJ=AI(JM)
      AJJ=AJ(JM)
      G=E/(2.*(1.+PR))
c     Member stiffness matrix in global coordinates , Eq.23-28 of Ghali
c     and Neville.
      DO 20 I=1,6
      DO 20 J=1,6
20    EST(I,J)=0.0
      EST(1,1)=12.*E*AIJ/(ALJ*ALJ*ALJ)
      EST(2,1)=-6.*E*AIJ*SIN/(ALJ*ALJ)
      EST(3,1)=6.*E*AIJ*C/(ALJ*ALJ)
      EST(4,1)=-EST(1,1)
      EST(5,1)=EST(2,1)
      EST(6,1)=EST(3,1)
      EST(2,2)=(G*AJJ*C*C+4.*E*AIJ*SIN*SIN)/ALJ
      EST(3,2)=(G*AJJ-4.*E*AIJ)*C*SIN/ALJ
      EST(4,2)=-EST(2,1)
      EST(5,2)=(-G*AJJ*C*C+2.*E*AIJ*SIN*SIN)/ALJ
      EST(6,2)=(-G*AJJ-2.*E*AIJ)*C*SIN/ALJ
      EST(3,3)=(G*AJJ*SIN*SIN+4.*E*AIJ*C*C)/ALJ
      EST(4,3)=-EST(3,1)
      EST(5,3)=EST(6,2)
      EST(6,3)=(-G*AJJ*SIN*SIN+2.*E*AIJ*C*C)/ALJ
      EST(4,4)=EST(1,1)
      EST(5,4)=-EST(2,1)
      EST(6,4)=-EST(3,1)
      EST(5,5)=EST(2,2)
      EST(6,5)=EST(3,2)
      EST(6,6)=EST(3,3)
      DO 30 I=1,6
      DO 30 J=I+1,6
30    EST(I,J)=EST(J,I)
c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE ESTFGR(ALJ,AIJ,AJJ,E,PR,ES)
c     Element stiffness matrix in local coordinates;member of grid;
c     deformations due to bending and torsion considered(Eq. 23-12,Ghali
c     and Neville).
c     ALJ=member length;AIJ=I;AJJ=J;PR=Poisson's ratio.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ES(6,6)
      G=E/(2.*(1.+PR))
      DO 10 I=1,6
      DO 10 J=1,6
10    ES(I,J)=0.0
      ES(1,1)=12.*E*AIJ/(ALJ*ALJ*ALJ)
      ES(3,1)=6.*E*AIJ/(ALJ*ALJ)
      ES(4,1)=-ES(1,1)
      ES(6,1)=ES(3,1)
      ES(2,2)=G*AJJ/ALJ
      ES(5,2)=-ES(2,2)
      ES(3,3)=4.*E*AIJ/ALJ
      ES(4,3)=-ES(3,1)
      ES(6,3)=2.*E*AIJ/ALJ
      ES(4,4)=ES(1,1)
      ES(6,4)=-ES(3,1)
      ES(5,5)=ES(2,2)
      ES(6,6)=ES(3,3)
      DO 20 I=1,6
      DO 20 J=I+1,6
20    ES(I,J)=ES(J,I)
      RETURN
      END

      SUBROUTINE LOADPF(JS,JE,X,Z,F,LR,AR,NJ3,NLR,NAR,NLC)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),X(NJ),Z(NJ),F(NJ3,NLC),LR(2,NLR)
      DIMENSION AR(6,NAR)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of load cases;
c     the line must be completed by another integer and 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,1X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(1X,'Load case ',3X,'Node',6X,'Fy',12X,'Mx',12X,'Mz')
20    CONTINUE
      READ(5,*)LCASE,NODE,FY,AMX,AMZ
c     IF(LCASE.GT.NLC)GO TO 30
      WRITE(6,300)LCASE,NODE,FY,AMX,AMZ
300   FORMAT(1X,I5,6X,I5,3(1X,E13.5))
      IF(LCASE.GT.NLC)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1,LCASE)=F(N1,LCASE)-FY
      F(N2,LCASE)=F(N2,LCASE)-AMX
      F(N3,LCASE)=F(N3,LCASE)-AMZ
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     or due to temperature,etc..Note that the last line in this set of
c     data lines must start with an integer > number of load cases;the
c     line must be completed by another integer and six real numbers.
c     The positive directions of the end forces are shown in Fig.23-2 of
c     Ghali and Neville.
      WRITE(6,400)
400   FORMAT(/,1X,'Member end forces with nodal displacement restrained
     *')
      WRITE(6,500)
500   FORMAT(1X,'Ld. case ',1x,'Member',2X,'Ar1',8X,'Ar2',8X,'Ar3',
     *8X,'Ar4',8X,'Ar5',8X,'Ar6')
      NRAR=0
c     NRAR is number of rows in [AR] of fixed-end forces.
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),AR(5,N),
     *AR(6,N)
      WRITE(6,600)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),
     *AR(5,N),AR(6,N)
600   FORMAT(1X,I5,1X,I5,1X,5(E10.4,1x),E10.4)
      IF(LR(1,N).GT.NLC)GO TO 50
      LCASE=LR(1,N)
      MEMBER=LR(2,N)
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces.To do this,identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      N1=3*JS(MEMBER)-2
      N2=N1+1
      N3=N1+2
      ALX=X(JE(MEMBER))-X(JS(MEMBER))
      ALZ=Z(JE(MEMBER))-Z(JS(MEMBER))
      AL=DSQRT(ALX*ALX+ALZ*ALZ)
      C=ALX/AL
      SIN=ALZ/AL
      F(N2,LCASE)=F(N2,LCASE)+AR(2,N)*C-AR(3,N)*SIN
      F(N3,LCASE)=F(N3,LCASE)+AR(2,N)*SIN+AR(3,N)*C
      F(N1,LCASE)=F(N1,LCASE)+AR(1,N)
      N1=3*JE(MEMBER)-2
      N2=N1+1
      N3=N1+2
      F(N2,LCASE)=F(N2,LCASE)+AR(5,N)*C-AR(6,N)*SIN
      F(N3,LCASE)=F(N3,LCASE)+AR(5,N)*SIN+AR(6,N)*C
      F(N1,LCASE)=F(N1,LCASE)+AR(4,N)
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1
      RETURN
      END

      SUBROUTINE SUPTGR(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 23-39 of Ghali and Neville. Note that before the adjustment,
c     [S] generated by the subroutine STIFGR is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION S(NJ3,NB),F(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC),SK(NS3)
      DO 10 I=1,NS3
      SK(I)=0.0
      DO 10 J=1,NLC
10    R(I,J)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eq. 23-
c     38 of Ghali and Neville.
      DO 50 I=1,NSJ
      M1=3*I-3
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)
      S(N,1)=S(N,1)*1.0E6
      DO 20 J=1,NLC
      R(M,J)=F(N,J)
      F(N,J)=-SK(M)*1.0E6*PD(I,K)
20    CONTINUE
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTGR(D,NS,ISUP,PD,R,SK,X,Z,JS,JE,AI,AJ,LR,AR,NLR,
     *NAR,NJ3,NS3,NLC)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION D(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC)
      DIMENSION SK(NS3),X(NJ),Z(NJ),JS(NM),JE(NM),AI(NM)
      DIMENSION AJ(NM),LR(2,NLR),AR(6,NAR)
      DIMENSION ES(6,6)
      WRITE(6,1000)
1000  FORMAT(1X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(1X,'Analysis results')
      DO 60 K=1,NLC
      WRITE(6,100)K
100   FORMAT(/,1X,'Load case no.',I5)
      WRITE(6,200)
200   FORMAT(/,1X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'w',12X,'(theta)x',8x,'(theta)z')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,D(N1,K),D(N2,K),D(N3,K)
400   FORMAT(1X,I5,3(3X,E13.5))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,1X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fy',14X,'Mx',14X,'Mz')
      DO 30 L=1,NSJ
      M1=3*L-3
      N1=3*NS(L)-3
      DO 20 J=1,3
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      R(M,K)=R(M,K)+SK(M)*1.0E6*(PD(L,J)-D(N,K))+SK(M)*D(N,K)
20    CONTINUE
      M1=M1+1
      M2=M1+1
      M3=M1+2
      WRITE(6,400)NS(L),R(M1,K),R(M2,K),R(M3,K)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,1X,'Member end forces')
      WRITE(6,800)
800   FORMAT(1X,'Member',7X,'F1',10X,'F2',10x,'F3',10x,'F4',10X,'F5',
     *10X,'F6')
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALZ*ALZ)
      C=ALX/ALJ
      SIN=ALZ/ALJ
      AIJ=AI(JM)
      AJJ=AJ(JM)
      CALL ESTFGR(ALJ,AIJ,AJJ,E,PR,ES)
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      N4=3*JE(JM)-2
      N5=N4+1
      N6=N4+2
c     Transform displacements at member ends into local coordinates.
      DT2=D(N2,K)*C+D(N3,K)*SIN
      DT3=-D(N2,K)*SIN+D(N3,K)*C
      DT1=D(N1,K)
      DT5=D(N5,K)*C+D(N6,K)*SIN
      DT6=-D(N5,K)*SIN+D(N6,K)*C
      DT4=D(N4,K)
c     For each member,calculate the product of its stiffness matrix by
c     its displacement vector.
      A1=ES(1,1)*DT1+ES(1,3)*DT3+ES(1,4)*DT4+ES(1,6)*DT6
      A2=ES(2,2)*DT2+ES(2,5)*DT5
      A3=ES(3,1)*DT1+ES(3,3)*DT3+ES(3,4)*DT4+ES(3,6)*DT6
      A4=ES(4,1)*DT1+ES(4,3)*DT3+ES(4,4)*DT4+ES(4,6)*DT6
      A5=ES(5,2)*DT2+ES(5,5)*DT5
      A6=ES(6,1)*DT1+ES(6,3)*DT3+ES(6,4)*DT4+ES(6,6)*DT6
c     Add the fixed-end forces included in the input data.
      IF(NRAR.EQ.0)GO TO 45
      DO 40 I=1,NRAR
      IF(LR(1,I).NE.K) GO TO 35
      IF(LR(2,I).NE.JM) GO TO 35
      A1=A1+AR(1,I)
      A2=A2+AR(2,I)
      A3=A3+AR(3,I)
      A4=A4+AR(4,I)
      A5=A5+AR(5,I)
      A6=A6+AR(6,I)
35    CONTINUE
40    CONTINUE
45    CONTINUE
      WRITE(6,900)JM,A1,A2,A3,A4,A5,A6
900   FORMAT(1X,I5,6(1X,E11.5))
50    CONTINUE
      WRITE(6,1200)
1200  FORMAT(1X,'-------------------------------------------------------
     *------------------------------------------------')
60    CONTINUE
      RETURN
      END
c_______________________________________________________________________

c		  Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 23-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------
c			  Program Listing
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 23-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S[HUGE](NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 23-49 to 23-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 23-48), determine [g]
c     and store in [D].
      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)
c     By Eq. 23-46 of Ghali and Neville and backward substitution(Eq.23-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,1X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,1X,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,1X,
     +'on the diagonal in solution of equations on row:',I4)
      STOP
99    CONTINUE
      RETURN
      END



