c			 Computer Program SPACEF
c			 Analysis of Space Frames
c                          January 2016



c			      by A. Ghali
c                 Emeritus Professor of Civil Engineering
c			The University of Calgary
c			  Calgary,Canada T2N 1N4

c Based on Ghali,A., Neville,A. M., Structural Analysis :A Unified Classical
c and Matrix Approach, CRC Press, Spon Press, 7th edition, 2016, Chapter 23
c-----------------------------------------------------------------------------
c				 Scope

c   SPACEF analyses space frames with straight prismatic members. Axial,
c bending and torsional de formations are considered; but deformations due
c to shear forces are ignored. The analysis gives the nodal displacements,
c the reactions and member end forces.
c The loading can be any, or cmbination,of the following types:
c (a) Three concentrated forces, and couples {Fx,Fy,Fz,Mx,My,Mz} at a node.
c (b) General load or temperature variation in a member, in which case
c     the twelve member fixed-end forces ,{Ar} must be included in the input.

c                              Sign Convention
c     Positive directions of nodal displacements{u,v,w,(theta)x,)(theta)y,
c  (theta)z} and member end forces are defined in Figs. 23-1 and 23-2 of
c  Ghali and Neville.The reaction components are determined in the directions of
c  the global axes, x,y and z. The three axes form a right-hand orthogonal
c  triad  (e.g. when the x axis is  horizontal pointing  to the right and the y axis is
c  vertical downwards , the z axis must be perpindicular to the page pointing
c  away from the reader)

c			  Input Data Instructions

c    The data, in free format, must be in a file named SPACEF.IN.The data is
c   composed of single lines or  sets of lines as follows:
c - Single line of title, not exceeding 72 characters.
c - Single line of 4 integers: number of nodes, number of members, number
c   of joints with prescribed displacement(s) and number of load cases.
c - Single line of two real values for E and Poisson's ratio.
c - A set of lines ;their number equals number of nodes. Each line is
c   composed of a node number followed by its x,y and z coordinates.
c - A set of lines; their number equals number of members. Each line
c   starts with 3 integers: a member number, its starting node and its end node,
c   node, followed by four real values: a, Iy, Iz, J, the cross- sectional area,
c   area, the second moments of area about centroidal principal axes y*
c   and z* and the  torsion constant,respectively.
c - A set of lines. Each line starts with an integer:a member number
c   followed by three real values:(lamda)y*x, (lamda)y*y and (lamda)y*z,
c   cosine of the angles between local axis y* and global directions x, y
c   and z, respectively. The set ends by a dummy line having an integer
c   greater the number of members followed by three dummy real numbers.
c   For any member not included in this set, the coputer assumes that the
c   y* axis is parallel to the global xy plane; thus Lamda y*z= 0; the
c   local axis z* is chosen such that Lamda z*z is positive; in practice, 
c   the xy plane is horizontal and all members can have their y* axis
c   horizont, in which case, only the dummy line is needed.In the special
c   case when the x* axis is in the same direction as the positive z-axis,
c   the z* axis is assumed along the negative x axis. When the x* axis is
c   in the direction of the negative z-axis, the z* axis is assumed along
c   the x axis.
c - A set of lines; their number equals number of joints with prescribed
c   displacements . Each line starts with a node number and six  restraint
c   indicators (0 or 1), followed by six real values of prescribed
c   displacements. The restraint indicators 1 and  0 mean free and
c   prescribed displacement component, respectively. When a displacement
c   component is free, a dummy value must be filled in for the prescribed
c   displacement (any real number)
c - A set of any number of lines for loading of type (a). Each line is
c   composed of two integers, a load case number and a node number,
c   followed by six real values of nodal forces: Fx, Fy, Fz, Mx, My, Mz.
c   The last line of the set must start with an integer greater than the
c   number of load cases followed by a dummy integer and six dummy real
c   numbers. The last line must not be omitted even when there is no
c   loading of type (a).
c - A set of any number of pairs of lines for loading of type(b). The first
c   line of each pair is composed of two integers:a load case number and a
c   member number, followed by six real values of member end forces; the
c   remaining six end forces are to be listed on the second line of the
c   pair. The last pair of the set must  have its first line starting with
c   an integer greater than the number of load cases, followed be a dummy
c   integer and six dummy real values; the secondt line of the pair
c   contains six dummy real values. The last pair of lines of the set must
c   not be omitted even when there is no loading of type (b).

c     Note that the prescribed displacement data apply to all load cases. If
c   this is not what is required the analysis for asupport movement is to be
c   done separately. 

c    Examle input data:

c  L-shaped horizl. cantlr., uniform gravity ld., square section
c  3  2  1  1                             NJ,NM,NSJ,NLC
c  1.0     0.2                            Elasticity modulus,Poisson's r.

c  1       0.      0.       0.            Node no.,x,y and z coords.
c  2       1.0     0.0      0.0
c  3       1.0     0.5      0.0

c  1  1  2 1.0e-2  8.33e-6  8.33e-6  14.06e-6  Member no.,JS,JE,a,Iy,Iz and J
c  2  2  3 1.0e-2  8.33e-6  8.33e-6  14.06e-6

c  10      0.0     0.0      0.0  Dummy: Member no.,Lamday*x,Lamday*y,Lamday*z

c  1  0  0  0  0  0  0  0. 0. 0. 0. 0. 0. Node no.,restr. indics.,prsc. dspls.

c  1     3   2.   0.  0.  0.  0.  0.        Ld. case no.,node,Fx,Fy,Fz,Mx,My,Mz
c  10    0   0.   0.  0.  0.  0.  0.        Dummy,end of data of loaded nodes

c  1  1  0.  0.  -0.5  0.  83.3e-3   0.   Ld. case no.,member,Ar1 to Ar6,
c        0.  0.  -0.5  0. -83.3e-3   0.   Ar6 to Ar12 (uniform gravity load
c  1  2  0.  0.  -0.25 0.  20.8e-3   0.   on members 1 and 2)
c        0.  0.  -0.25 0. -20.8e-3   0.

c  10 0  6*0.                             Dummy,end of data of member loads
c        6*0.   

c    When it is desired to ignore axial or torsion deformations ,enter large
c  values for a or J. To run the program, type SPACEF and press Enter;
c  the results will be in a file named SPACEF.OUT.

c			     Limits on Data

c    Vectors {B} and {NI} are used to store all the real and the integer
c  arrays used by the program. The maximum dimensions set are: MAXB=40000 and MAXNI=
c  2000 . This would normally be sufficient for a frame approximately 120
c  nodes, 120 members and 8 load cases, depending upon the band width. The
c  maximum dimensions may be changed as required on line 2 of program(see
c  listing below). After the change, the program SPACEF.FOR and EQSOLN.FOR
c  (equations solver) must be compiled using a fortran compiler(e.g.
c  Microsoft). The length of the vectors used for a given problem may be
c  calculated by:

c  Dimension of {B}=NJ*(3+12*NLC+6*NB)+7*NM+6*NSJ*(NLC+2)+12*(NRAR+1)

c  Dimension of{NI}=2*NM+7*NSJ+2*(NRAR+1)
c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c                                                                  rows  cols.
c  {A}    cross-section areas                                      NM
c  {AIY}  second moments of areas of cross-sections                NM
c  {AIZ}  second moments of areas of cross sections
c  {AJ}   torsion constants of member cross-sections               NM
c  {ARD}  effective shear cross-section areas                      NM
c  [AR]   member end forces with nodal displacements restrained    12    NRAR
c  [CYL]  direction cosines of local axis y* .Each row contains    NM    3
c	  (lamda)y*x,(lamda)y*y,(lamda)y*z, which are cosine the
c	  angles between y* and global x,y and z axes.
c  [D]    nodal displacements                                      6*NJ  NLC
c   E	  modulus of elasticity
c  [ES]   element (member) stiffness in local coordinates          12    12
c  [EST]  element stiffness matrix in directions parallel to       12    12
c	  global axes
c  [F]    nodal restraining forces                                 6*NJ  NLC
c  [ISUP] joint restraint indicators : 1 and o ,indicating free    NSJ   6
c	  and prescribed displacement respectively
c  {JE}   end nodes of individual members                          NM
c  {JS}   starting node of individual members                      NM
c  [LR]   list of load case and member number for which fixed-end  2     NRAR
c	  forces are given as data
c  NB     band width ; see Sec. 23-3 of Ghali and Neville
c  NJ	  number of joints (nodes)
c  NLC	  number of load cases
c  NM	  number of members
c  NRAR   number of rows in [AR] ; this is equal the number of
c	  members for which fixed-end forces are given as data
c  {NS}   list of node numbers where supports are provided or      NSJ
c	  displacements are prescribed
c  NSJ	  number of supprted joints (nodes with prescribed displac-
c	  ements)
c  [PD]   prescribed displacement values                           NSJ   6
c  PR	  Poisson's ratio
c  [R]    forces at the supprted nodes including the reactions     6*NSJ NLC
c  [S]    stiffness matrix                                         6*NJ  NB
c  {SK}   diagonal elements of [S] corresponding to  coordinates   6*NSJ
c	  at the supportd nodes
c  {X}    x coordinate of nodes                                    NJ
c  {Y}    y coordinates of nodes                                   NJ
c  {Z}    z coordinates of nodes                                   NJ
c------------------------------------------------------------------------------
c	    Listing of program SPACEF
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=40000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='SPACEF.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='SPACEF.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(2X,A76)
      READ(5,*)NJ,NM,NSJ,NLC
      WRITE(6,300)NJ
300   FORMAT(2X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(2X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(2X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NLC
600   FORMAT(2X,'Number of load cases=',I5)
      NDF=6*NJ
      NS6=6*NSJ
c     Elements of {B} at which real arrays start are:
c     {A}:N1  {AIY}:N2	{AIZ}:N3 {AJ}:N4  [CYL]:N5 [D]:N6  [F]:N7
c     [PD]:N8 [R]:N9 {SK}:N10 {X}:N11  {Y}:N12	{Z}:N13  [S]:N14
c     [AR]:N15
      N1=1
      N2=N1+NM
      N3=N2+NM
      N4=N3+NM
      N5=N4+NM
      N6=N5+3*NM
      N7=N6+NDF*NLC
      N8=N7+NDF*NLC
      N9=N8+NS6
      N10=N9+NS6*NLC
      N11=N10+NS6
      N12=N11+NJ
      N13=N12+NJ
      N14=N13+NJ
c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  [LR]:K5
      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      K5=K4+NS6
c     CALL DATASF(X,Y,Z,JS,JE,A,AIY,AIZ,AJ,CYL,NS,ISUP,PD,NB)
      CALL DATASF(B(N11),B(N12),B(N13),NI(K1),NI(K2),B(N1),B(N2),B(N3),
     *B(N4),B(N5),NI(K3),NI(K4),B(N8),NB)
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N15=N14+NDF*NB
c     Determine maximum number of columns of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     columns of [R] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N15)/12
      NLR=(MAXNI-K5)/2
c     CALL STIFSF(JS,JE,A,AIY,AIZ,AJ,CYL,X,Y,Z,S,NDF,NB)
      CALL STIFSF(NI(K1),NI(K2),B(N1),B(N2),B(N3),B(N4),B(N5),B(N11),
     *B(N12),B(N13),B(N14),NDF,NB)
c     CALL LOADSF(JS,JE,X,Y,Z,CYL,F,LR,AR,NDF,NLR,NAR,NLC)
      CALL LOADSF(NI(K1),NI(K2),B(N11),B(N12),B(N13),B(N5),B(N7),
     *NI(K5),B(N15),NDF,NLR,NAR,NLC)
c     CALL SUPTSF(S,F,NS,ISUP,PD,R,SK,NDF,NS6,NB,NLC)
      CALL SUPTSF(B(N14),B(N7),NI(K3),NI(K4),B(N8),B(N9),B(N10),NDF,
     *NS6,NB,NLC)
c     CALL EQSOLN(S,F,D,NDF,NB,NLC)
      CALL EQSOLN(B(N14),B(N7),B(N6),NDF,NB,NLC)
c     CALL RSLTSF(D,NS,ISUP,PD,R,SK,X,Y,Z,JS,JE,A,AIY,AIZ,AJ,CYL,LR,AR,
c    *NLR,NAR,NDF,NS6,NLC)
      CALL RSLTSF(B(N6),NI(K3),NI(K4),B(N8),B(N9),B(N10),B(N11),B(N12),
     *B(N13),NI(K1),NI(K2),B(N1),B(N2),B(N3),B(N4),B(N5),NI(K5),B(N15),
     *NLR,NAR,NDF,NS6,NLC)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATASF(X,Y,Z,JS,JE,A,AIY,AIZ,AJ,CYL,NS,ISUP,PD,NB)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION X(NJ),Y(NJ),Z(NJ),JS(NM),JE(NM),A(NM),AIY(NM),AIZ(NM)
      DIMENSION AJ(NM),CYL(NM,3),NS(NSJ),ISUP(NSJ,6),PD(NSJ,6)
      READ(5,*)E,PR
      WRITE(6,200)E,PR
200   FORMAT(2X,'Elasticity modulus=',E13.5,2X,';Poissons ratio=',F6.4)
      WRITE(6,300)
300   FORMAT(/,2X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',6x,'x',9X,'y',9X,'z')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J),Z(J)
10    WRITE(6,500)J,X(J),Y(J),Z(J)
500   FORMAT(2X,I5,3F10.3)
      WRITE(6,600)
600   FORMAT(/,2X,'Element information')
      WRITE(6,700)
700   FORMAT(2X,'Element',2X,'1st node',3X,'2nd node',5x,'a',11x,'Iy',
     *11x,'Iz',11X,'J')
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),A(J),AIY(J),AIZ(J),AJ(J)
c     Member no.,starting node,end node,cross-section area,2nd moment
c     of area about centroidal axis parallel to local y* , 2nd moment
c     of area about centroidal axis parallel to local z* and torsion
c     constant.
20    WRITE(6,800)J,JS(J),JE(J),A(J),AIY(J),AIZ(J),AJ(J)
800   FORMAT(2X,I5,3X,I5,5X,I5,2X,4(E12.5,1X))
      WRITE(6,900)
900   FORMAT(/,3X,'Cosine of angles between local y* and global axes')
      WRITE(6,1000)
1000  FORMAT(2X,'Element',4X,'(lamda)y*x',4X,'(lamda)y*y',4X,'(lamda)y*z
     *')

2001  CONTINUE
      READ(5,*)J,Var1,Var2,Var3
      IF (J.GT.NM)GO TO 2002
      IF(J.LE.NM)CYL(J,1)=Var1
      IF(J.LE.NM)CYL(J,2)=Var2
      IF(J.LE.NM)CYL(J,3)=Var3
      GO TO 2001
2002  CONTINUE

      DO 2004 J=1,NM
      CHECK=SQRT(CYL(J,1)**2+ CYL(J,2)**2+CYL(J,3)**2)
      IF(CHECK.EQ.0.0)THEN
      NNS=JS(J)
      NNE=JE(J)
      CALL ystar
     +(X(NNS),Y(NNS),Z(NNS),X(NNE),Y(NNE),Z(NNE),
     +C1,C2,C3)
      CYL(J,1)=C1
      CYL(J,2)=C2
      CYL(J,3)=C3
      END IF
2004  CONTINUE
      DO 2006 J=1,NM
      WRITE(6,1100)J,CYL(J,1),CYL(J,2),CYL(J,3)
1100  FORMAT(2X,I5,3(6X,F8.5))
2006  CONTINUE
      DO 2007 J=1,NM
      CHECK=(CYL(J,1)**2+ CYL(J,2)**2+CYL(J,3)**2)
      IF(ABS(1.-CHECK).GE.10.e-6)WRITE(6,2005)J,(CHECK-1.)
2005  FORMAT(/,2X,'ERROR: For member',I5,
     +', sum of squares of direction cosines-1=',E13.5,
     +/,2X,'This value should be zero.')
2007  CONTINUE

1200  FORMAT(2X,I5,3X,3(E13.5,1X))
      WRITE(6,1300)
1300  FORMAT(/,2X,'Support conditions')
      WRITE(6,1400)
1400  FORMAT(2X,'Node',5X,'Restraint indicators',9X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or 0,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1500)
1500  FORMAT(9X,'u',3x,'v',3X,'w',2X,'thx',1x,'thy',1X,'thz',
     *4X,'u',7X,'v',7X,'w',4X,'thetax',2X,'thetay',2X,'thetaz')
1600  FORMAT(2X,I5,I3,5(1X,I3),6(1X,F7.4))
      DO 40 J=1,NSJ
      READ(5,*)NS(J),(ISUP(J,K),K=1,6),(PD(J,K),K=1,6)
40    WRITE(6,1600)NS(J),(ISUP(J,K),K=1,6),(PD(J,K),K=1,6)
c     Determine band width by Eq. 23-1 of Ghali and Neville
      NBI=0
      DO 60 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 50
      N=-N
50    CONTINUE
      IF(N.GT.NBI)NBI=N
60    CONTINUE
      NB=6*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFSF(JS,JE,A,AIY,AIZ,AJ,CYL,X,Y,Z,S,NDF,NB)
c     Stiffness matrix for space frame.See Sec. 23-9,Ghali and Neville
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),A(NM),AIY(NM),AIZ(NM),AJ(NM),CYL(NM,3)
      DIMENSION X(NJ),Y(NJ),Z(NJ),S(NDF,NB)
      DIMENSION ES(12,12),IM(12),T(3,3),TT(3,3),H(3,3),P(3,3)
      DO 10 I=1,NDF
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 80 JM=1,NM
c     Generate a 3X3 transformation matrix [t], Eq. 23-18 of Ghali and Neville

      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      T(1,1)=ALX/ALJ
      T(1,2)=ALY/ALJ
      T(1,3)=ALZ/ALJ
      T(2,1)=CYL(JM,1)
      T(2,2)=CYL(JM,2)
      T(2,3)=CYL(JM,3)
      T(3,1)=T(1,2)*T(2,3)-T(1,3)*T(2,2)
      T(3,2)=T(1,3)*T(2,1)-T(1,1)*T(2,3)
      T(3,3)=T(1,1)*T(2,2)-T(1,2)*T(2,1)

c     Generate the transpose of [t].
      DO 20 I=1,3
      DO 20 J=1,3
20    TT(I,J)=T(J,I)
      AAJ=A(JM)
      AIYJ=AIY(JM)
      AIZJ=AIZ(JM)
      AJJ=AJ(JM)




c   Member stiffness matrix in local coordinates, Eq.6-6 of Ghali and Neville

      CALL ESTFSF(ALJ,AAJ,AIYJ,AIZJ,AJJ,E,PR,ES)

c     Transform element stiffness,[ES]from local to global directions;
c     see Eq. 23-19 of Ghali and Neville. Store the result back in [ES].
c     Partition [ES] into 3x3 submatrices and postmultipy each by [t]
c     and store the product in [P]. Postmultiply [P] by the transpose
c     of [t]. Store the resulting submatrices back in [ES].
      DO 60 N=1,4
      DO 60 L=1,4
      DO 40 I=1,3
      DO 40 J=1,3
      IES=3*N-3+I
      JES=3*L-3+J
      H(I,J)=ES(IES,JES)
40    CONTINUE
      CALL MLTPLY(H,T,P,3,3,3)
      CALL MLTPLY(TT,P,H,3,3,3)
      DO 50 I=1,3
      DO 50 J=1,3
      IES=3*N-3+I
      JES=3*L-3+J
      ES(IES,JES)=H(I,J)
50    CONTINUE
60    CONTINUE


c     Identify the 12 coordinates at the start and end of member.
      IM(1)=6*JS(JM)-5
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=IM(1)+3
      IM(5)=IM(1)+4
      IM(6)=IM(1)+5
      IM(7)=6*JE(JM)-5
      IM(8)=IM(7)+1
      IM(9)=IM(7)+2
      IM(10)=IM(7)+3
      IM(11)=IM(7)+4
      IM(12)=IM(7)+5
      DO 70 I=1,12
      DO 70 J=1,12
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+ES(I,J)
35    CONTINUE
70    CONTINUE
80    CONTINUE

      RETURN
      END

      SUBROUTINE ESTFSF(ALJ,AAJ,AIYJ,AIZJ,AJJ,E,PR,ES)
c     Element stiffness matrix in local coordinates;member of space
c     frame,(Eq. 6-6, Ghali and Neville).
c     ALJ=member length;AJ=a;AIJ=I;ARDJ=ar;PR=Poisson's ratio.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION ES(12,12)
      G=E/(2.*(1.+PR))
      DO 10 I=1,12
      DO 10 J=1,12
10    ES(I,J)=0.0
      ES(1,1)=E*AAJ/ALJ
      ES(7,1)=-ES(1,1)
      ES(7,7)=ES(1,1)
      ES(2,2)=12.0*E*AIZJ/(ALJ*ALJ*ALJ)
      ES(8,2)=-ES(2,2)
      ES(8,8)=ES(2,2)
      ES(3,3)=12.0*E*AIYJ/(ALJ*ALJ*ALJ)
      ES(9,3)=-ES(3,3)
      ES(9,9)=ES(3,3)
      ES(4,4)=G*AJJ/ALJ
      ES(10,4)=-ES(4,4)
      ES(10,10)=ES(4,4)
      ES(6,2)=6.0*E*AIZJ/(ALJ*ALJ)
      ES(12,2)=ES(6,2)
      ES(12,8)=-ES(6,2)
      ES(5,3)=-6.0*E*AIYJ/(ALJ*ALJ)
      ES(11,3)=ES(5,3)
      ES(11,9)=-ES(11,3)
      ES(9,5)=6.0*E*AIYJ/(ALJ*ALJ)
      ES(11,5)=2.0*E*AIYJ/ALJ
      ES(8,6)=-6.0*E*AIZJ/(ALJ*ALJ)
      ES(12,6)=2.0*E*AIZJ/ALJ
      ES(5,5)=4.0*E*AIYJ/ALJ
      ES(11,11)=ES(5,5)
      ES(6,6)=4.0*E*AIZJ/ALJ
      ES(12,12)=ES(6,6)
      DO 20 I=1,12
      DO 20 J=I+1,12
20    ES(I,J)=ES(J,I)
      RETURN
      END

      SUBROUTINE LOADSF(JS,JE,X,Y,Z,CYL,F,LR,AR,NDF,NLR,NAR,NLC)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION JS(NM),JE(NM),X(NJ),Y(NJ),Z(NJ),CYL(NM,3),F(NDF,NLC)
      DIMENSION LR(2,NLR),AR(12,NAR),TT(3,3)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of load cases;
c     the line must be completed by another integer and 6 real numbers.
      WRITE(6,100)
100   FORMAT(/,2X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(2X,'Case ',1X,'Node',6X,'Fx',9X,'Fy',9X,'Fz',
     *9X,'Mx',9X,'My',9X,'Mz')
20    CONTINUE
      READ(5,*)LCASE,NODE,FX,FY,FZ,AMX,AMY,AMZ
c     IF(LCASE.GT.NLC)GO TO 30
      WRITE(6,300)LCASE,NODE,FX,FY,FZ,AMX,AMY,AMZ
300   FORMAT(2X,I4,2X,I4,6(1X,E10.4))
      IF(LCASE.GT.NLC)GO TO 30
      N1=6*NODE-5
      N2=N1+1
      N3=N2+1
      N4=N3+1
      N5=N4+1
      N6=N5+1
      F(N1,LCASE)=F(N1,LCASE)-FX
      F(N2,LCASE)=F(N2,LCASE)-FY
      F(N3,LCASE)=F(N3,LCASE)-FZ
      F(N4,LCASE)=F(N4,LCASE)-AMX
      F(N5,LCASE)=F(N5,LCASE)-AMY
      F(N6,LCASE)=F(N6,LCASE)-AMZ
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to loading away from the nodes
c     or due to temperature,etc..Note that the penultimate line in this
c     data set must start with an integer > number of load cases;the
c     line must be completed by another integer and six real numbers.
c     The last line in the set must contain six dummy real values.
c     The positive directions of the end forces are shown in Fig.23-2 of
c     Ghali and Neville.
      WRITE(6,400)
400   FORMAT(/,2X,'Member end forces with nodal displacement restrained
     *')
      WRITE(6,500)
500   FORMAT(4X,'Case ',1x,'Member',2X,'Ar1',8X,'Ar2',8X,'Ar3',
     *8X,'Ar4',8X,'Ar5',8X,'Ar6')
      WRITE(6,550)
550   FORMAT(18X,'Ar7',8X,'Ar8',8X,'Ar9',8X,'Ar10',7X,'Ar11',7X,'Ar12')
      NRAR=0
c     NRAR is number of rows in [AR];that is number of pairs of lines
c     of data for loading of type (b).The last pair of lines of data
c     is not included.
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),AR(5,N),
     *AR(6,N)
      WRITE(6,600)LR(1,N),LR(2,N),AR(1,N),AR(2,N),AR(3,N),AR(4,N),
     *AR(5,N),AR(6,N)
600   FORMAT(2X,I4,1X,I5,1X,5(E10.4,1x),E10.4)
      READ(5,*)AR(7,N),AR(8,N),AR(9,N),AR(10,N),AR(11,N),AR(12,N)
      WRITE(6,650)AR(7,N),AR(8,N),AR(9,N),AR(10,N),AR(11,N),AR(12,N)
650   FORMAT(13X,5(E10.4,1X),E10.4)
      IF(LR(1,N).GT.NLC)GO TO 50
      LCASE=LR(1,N)
      MEMBER=LR(2,N)
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces.To do this,identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      N1=6*JS(MEMBER)-5
      N2=N1+1
      N3=N1+2
      N4=N1+3
      N5=N1+4
      N6=N1+5
      ALX=X(JE(MEMBER))-X(JS(MEMBER))
      ALY=Y(JE(MEMBER))-Y(JS(MEMBER))
      ALZ=Z(JE(MEMBER))-Z(JS(MEMBER))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      TT(1,1)=ALX/AL
      TT(2,1)=ALY/AL
      TT(3,1)=ALZ/AL
      TT(1,2)=CYL(MEMBER,1)
      TT(2,2)=CYL(MEMBER,2)
      TT(3,2)=CYL(MEMBER,3)
      TT(1,3)=TT(2,1)*TT(3,2)-TT(3,1)*TT(2,2)
      TT(2,3)=TT(3,1)*TT(1,2)-TT(1,1)*TT(3,2)
      TT(3,3)=TT(1,1)*TT(2,2)-TT(2,1)*TT(1,2)
c    Apply Eq. 21-22 of Ghali and Neville add the result to the existing load
c     vector [F].
      F(N1,LCASE)=F(N1,LCASE)+AR(1,N)*TT(1,1)+AR(2,N)*TT(1,2)+AR(3,N)*
     *TT(1,3)
      F(N2,LCASE)=F(N2,LCASE)+AR(1,N)*TT(2,1)+AR(2,N)*TT(2,2)+AR(3,N)*
     *TT(2,3)
      F(N3,LCASE)=F(N3,LCASE)+AR(1,N)*TT(3,1)+AR(2,N)*TT(3,2)+AR(3,N)*
     *TT(3,3)
      F(N4,LCASE)=F(N4,LCASE)+AR(4,N)*TT(1,1)+AR(5,N)*TT(1,2)+AR(6,N)*
     *TT(1,3)
      F(N5,LCASE)=F(N5,LCASE)+AR(4,N)*TT(2,1)+AR(5,N)*TT(2,2)+AR(6,N)*
     *TT(2,3)
      F(N6,LCASE)=F(N6,LCASE)+AR(4,N)*TT(3,1)+AR(5,N)*TT(3,2)+AR(6,N)*
     *TT(3,3)
c     Repeat for forces at end node of the member.
      N1=6*JE(MEMBER)-5
      N2=N1+1
      N3=N1+2
      N4=N1+3
      N5=N1+4
      N6=N1+5
      F(N1,LCASE)=F(N1,LCASE)+AR(7,N)*TT(1,1)+AR(8,N)*TT(1,2)+AR(9,N)*
     *TT(1,3)
      F(N2,LCASE)=F(N2,LCASE)+AR(7,N)*TT(2,1)+AR(8,N)*TT(2,2)+AR(9,N)*
     *TT(2,3)
      F(N3,LCASE)=F(N3,LCASE)+AR(7,N)*TT(3,1)+AR(8,N)*TT(3,2)+AR(9,N)*
     *TT(3,3)
      F(N4,LCASE)=F(N4,LCASE)+AR(10,N)*TT(1,1)+AR(11,N)*TT(1,2)+
     *AR(12,N)*TT(1,3)
      F(N5,LCASE)=F(N5,LCASE)+AR(10,N)*TT(2,1)+AR(11,N)*TT(2,2)+
     *AR(12,N)*TT(2,3)
      F(N6,LCASE)=F(N6,LCASE)+AR(10,N)*TT(3,1)+AR(11,N)*TT(3,2)+
     *AR(12,N)*TT(3,3)
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1
      RETURN
      END

      SUBROUTINE SUPTSF(S,F,NS,ISUP,PD,R,SK,NDF,NS6,NB,NLC)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 23-39 of Ghali and Neville. Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION S(NDF,NB),F(NDF,NLC),NS(NSJ),ISUP(NSJ,6),PD(NSJ,6)
      DIMENSION R(NS6,NLC),SK(NS6)
      DO 10 I=1,NS6
      SK(I)=0.0
      DO 10 J=1,NLC
10    R(I,J)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eq. 23-38
c     of Ghali and Neville.
      DO 70 I=1,NSJ
      M1=6*I-6
      N1=6*NS(I)-6
      DO 40 K=1,6
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)
      S(N,1)=S(N,1)*1.0E6
      DO 20 J=1,NLC
      R(M,J)=F(N,J)
      F(N,J)=-SK(M)*1.0E6*PD(I,K)
20    CONTINUE
30    CONTINUE
40    CONTINUE
70    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTSF(D,NS,ISUP,PD,R,SK,X,Y,Z,JS,JE,A,AIY,AIZ,AJ,CYL,
     *LR,AR,NLR,NAR,NDF,NS6,NLC)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,PR,NRAR
      DIMENSION D(NDF,NLC),NS(NSJ),ISUP(NSJ,6),PD(NSJ,6)
      DIMENSION R(NS6,NLC)
      DIMENSION SK(NS6),X(NJ),Y(NJ),Z(NJ),JS(NM),JE(NM),A(NM),AIY(NM)
      DIMENSION AIZ(NM),CYL(NM,3),AJ(NM),LR(2,NLR),AR(12,NAR)
      DIMENSION ES(12,12),T(3,3),ND(12),DT(12),H(3),P(3),ARM(12)
      WRITE(6,1000)
1000  FORMAT(2X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(2X,'Analysis results')
      DO 95 K=1,NLC
      WRITE(6,100)K
100   FORMAT(/,2X,'Load case no.',I5)
      WRITE(6,200)
200   FORMAT(/,2X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',6X,'u',11X,'v',11X,'w',8X,'(theta)x',3X,
     *'(theta)y',3X,'(theta)z')
      DO 10 I=1,NJ
      N1=6*I-5
      N2=N1+1
      N3=N1+2
      N4=N1+3
      N5=N1+4
      N6=N1+5
      WRITE(6,400)I,D(N1,K),D(N2,K),D(N3,K),D(N4,K),D(N5,K),D(N6,K)
400   FORMAT(1X,I5,6(1X,E11.4))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,2X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'Node',6X,'Fx',10X,'Fy',10X,'Fz',10X,'Mx',10X,'My',10X,
     *'Mz')
      DO 30 L=1,NSJ
      M1=6*L-6
      N1=6*NS(L)-6
      DO 20 J=1,6
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      R(M,K)=R(M,K)+SK(M)*1.0E6*(PD(L,J)-D(N,K))+SK(M)*D(N,K)
20    CONTINUE
      M1=M1+1
      M2=M1+1
      M3=M1+2
      M4=M1+3
      M5=M1+4
      M6=M1+5
      WRITE(6,400)NS(L),R(M1,K),R(M2,K),R(M3,K),R(M4,K),R(M5,K),R(M6,K)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,2X,'Member end forces')
      WRITE(6,800)
800   FORMAT(2X,'Member',5X,'F1',10X,'F2',10x,'F3',10x,'F4',10X,'F5',
     *10X,'F6')
      WRITE(6,850)
850   FORMAT(13X,'F7',10X,'F8',10X,'F9',10X,'F10',9X,'F11',9X,'F12')
      DO 90 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      T(1,1)=ALX/ALJ
      T(1,2)=ALY/ALJ
      T(1,3)=ALZ/ALJ
      T(2,1)=CYL(JM,1)
      T(2,2)=CYL(JM,2)
      T(2,3)=CYL(JM,3)
      T(3,1)=T(1,2)*T(2,3)-T(1,3)*T(2,2)
      T(3,2)=T(1,3)*T(2,1)-T(1,1)*T(2,3)
      T(3,3)=T(1,1)*T(2,2)-T(1,2)*T(2,1)
      AAJ=A(JM)
      AIYJ=AIY(JM)
      AIZJ=AIZ(JM)
      AJJ=AJ(JM)
      CALL ESTFSF(ALJ,AAJ,AIYJ,AIZJ,AJJ,E,PR,ES)
      ND(1)=6*JS(JM)-5
      ND(2)=ND(1)+1
      ND(3)=ND(1)+2
      ND(4)=ND(1)+3
      ND(5)=ND(1)+4
      ND(6)=ND(1)+5
      ND(7)=6*JE(JM)-5
      ND(8)=ND(7)+1
      ND(9)=ND(7)+2
      ND(10)=ND(7)+3
      ND(11)=ND(7)+4
      ND(12)=ND(7)+5
c     Transform displacements at member ends into local coordinates.
      DO 40 M=1,4
      I1=3*M-2
      I2=I1+1
      I3=I1+2
      H(1)=D(ND(I1),K)
      H(2)=D(ND(I2),K)
      H(3)=D(ND(I3),K)
      CALL MLTPLY(T,H,P,3,3,1)
      DT(I1)=P(1)
      DT(I2)=P(2)
      DT(I3)=P(3)
40    CONTINUE
c     For each member,calculate the product of its stiffness matrix by
c     its displacement vector.Store result in {ARM}.
      CALL MLTPLY(ES,DT,ARM,12,12,1)
c     Add the fixed-end forces included in the input data to {ARM}.
      IF(NRAR.EQ.0)GO TO 45
      DO 70 I=1,NRAR
      IF(LR(1,I).NE.K) GO TO 35
      IF(LR(2,I).NE.JM) GO TO 35
      DO 60 J=1,12
      ARM(J)=ARM(J)+AR(J,I)
60    CONTINUE
35    CONTINUE
70    CONTINUE
45    CONTINUE
      WRITE(6,900)JM,(ARM(J),J=1,6)
900   FORMAT(1X,I5,6(1X,E11.5))
      WRITE(6,950)(ARM(J),J=7,12)
950   FORMAT(6X,6(1X,E11.5))
90    CONTINUE
      WRITE(6,1200)
1200  FORMAT(2X,'-------------------------------------------------------
     *------------------------------------------------')
95    CONTINUE
      RETURN
      END

      SUBROUTINE MLTPLY(A,C,P,K,M,N)
c     Matrix multiplication : [A][C]=[P]
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(K,M),C(M,N),P(K,N)
      DO 10 I=1,K
      DO 10 J=1,N
10    P(I,J)=0.0
      DO 20 L=1,N
      DO 20 I=1,K
      DO 20 J=1,M
      P(I,L)=P(I,L)+A(I,J)*C(J,L)
20    CONTINUE
      RETURN
      END

      SUBROUTINE ystar
     +(XS,YS,ZS,XE,YE,ZE,
     +C1,C2,C3)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c     Direction cosines of local y* axis.  The subroutine is used only
c     when the direction cosines are not given. In which case, the y*
c     axis is assumed horizontal and directed such that Lmdaz*z is positive
c     or zero.
      AL=SQRT((XE-XS)**2+(YE-YS)**2+(ZE-ZS)**2)
      A1=(XE-XS)/AL
      A2=(YE-YS)/AL
      A3=(ZE-ZS)/AL
      ALbar=SQRT(A1**2+A2**2)
      IF(ALbar.NE.0.0)C1=-A2/ALbar
      IF(ALbar.NE.0.0)C2=A1/ALbar
      IF(ALbar.NE.0.0)C3=0.
      IF(ABS(A3).EQ.1)C1=0.
      IF(ABS(A3).EQ.1)C2=1.
      IF(ABS(A3).EQ.1)C3=0.
      RETURN
      END

c		  Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c		   Based on Ghali,A. and Neville,A. M.,
c      Structural Analysis :A Unified Classical and Matrix Approach
c       CRC Press, Spon Press, 7th edition, 2016, Chapter 23
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 24-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------
c			  Program Listing
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)

c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 22-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S[HUGE](NDF,NB),F(NDF,NLC),D(NDF,NLC)

      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 24-49 to 24-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 24-48), determine [g]
c     and store in [D].
      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)
c     By Eq. 24-46 of Ghali and Neville and backward substitution(Eq.24-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,2X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,2X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,2X,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,2X,
     +'on the diagonal in solution of equations on row:',I4)
      STOP
99    CONTINUE
      RETURN
      END
