
c                        Computer Program SPACET
c                       Analysis of Space Trusses    
c                           January 2016

c			     by A. Ghali
c                Emeritus Professor of Civil Engineering
c                      The University of Calgary
c                       Calgary, Canada T2N 1N4

c  Based on Ghali,A., Neville,A. M., Structural Analysis :A Unified Classical
c  and Matrix Approach, CRC Press, Spon Press, 7th. ed.,2016, Chapter 23.
c___________________________________________________________________________

c				 Scope
c    SPACET analyses space trusses composed of pin-connected straight prism-
c  atic members. The analysis gives the nodal displacements, the reactions
c  and the member end forces.
c    The loading can be one or cmbination of the following types:
c  (a)Concentrated forces Fx,Fy and Fz in the direction of the global x,y
c     and z axes applied at the nodes.
c  (b)Temperature ,shrinkage, lack of fit ,etc. of individual members, in
c     which case the two member fixed-end forces {Ar}of the members must
c     be included in the input.
c
c			      Sign Convention
c     Positive directions of nodal displacements u,v and w are in the
c  positive directions of the global x,y and z axes. The reaction components
c  are determined in the same directions. The two member end forces are
c  positive when they are in the direction of local axis x* , which is along
c  the straight line joining the start and end of the member ( Fig. 23-1 and
c  23-2 of Ghali and Neville.
c
c			  Input Data Instructions
c     The data, in free format , must be in a file named SPACET.IN. Any editor
c  may be used. The data is composed of single lines or sets of lines as
c  follows:

c    --Single line of title ,not exceeding 72 characters.
c    --Single line of 4 integers: number of nodes,number of members,number of
c      joints with prescribed displacement(s) and number of load cases.
c    --Single line of one real values for E .
c    --A set of lines ;their number equals number of nodes.Each line is
c      composed of a node number followed by its x , y and z coordinates.
c    --A set of lines their number equals number of members.Each line starts
c      with 3 integers: a member number ,its starting node and its end node,
c      followed by the cross-section area, a.
c    --A set of lines their number equals number of joints with prescribed
c      displalacements . Each line starts with a node number and three
c      restraint indicators (0 or 1),followed by three real values of
c      prescribed displacements. The restraint indicators 1 and 0 mean free
c      and prescribed displacement component,respectively. When a displacement 
c      component is free, a dummy value must be filled in for the prescribed 
c      displacement  (any real number).
c    --A set of any number of lines for loading of type(a).Each line is 
c      composed of two integers ,a load case number and a node number,
c      followed by three real values of nodal forces Fx,Fy and Fz. The last 
c      line of the set must start by an integer greater than the number 
c      of the load cases followed by a dummy integer and three  dummy real
c      numbers. The last line must not be omitted even when there is no
c      loading of type (a).
c    --A set of any number of lines for loading of type(b).Each line is
c      composed of two integers,a load case number and a member number,
c      followed by two real values of member end forces. The last line of
c      this set must start with an integer greater than the number of load
c      cases followed by a dummy integer and two dummy real values. The last
c      line must not be omitted even when there is no loading of type (b).

c    Note that the prescribed displacement data apply to all load cases. If
c  this is not what is required ,the analysis for a support movement is to be
c  done separately. This restriction can be avoided by entering zero precribed
c  displacements and giving the member end forces when the prescribed displac-
c  ements occur while the other displacements are zero(See Example 23-2 of
c  Ghali and Neville.

c    Examle input data:

c Space truss. Problem 8-3  of Ghali and Neville
c  4	3    3	  1			   NJ,NM,NSJ,NLC
c  1.0					   Elasticity modulus
c  1	     0.0	 0.0	     0.0   Node no.,x,y and z coordinates.
c  2	     0.0	-0.5	     1.0
c  3	     0.433	 0.25	     1.0
c  4	    -0.433	 0.25	     1.0
c  1    1    2      1.0                  Member no.,JS,JE and cross-sec. area
c  2	1    3	    1.0
c  3	1    4	    1.0
c  2    0    0    0    0.0   0.0    0.0  Node no.,restr. indics.,prsc. displs.
c  3    0    0    0    0.0   0.0    0.0
c  4    0    0    0    0.0   0.0    0.0
c  1	1    1.0    0.0    0.0		   Ld. case no.,node,Fx,Fy and Fz
c 10	0    0.0    0.0    0.0		   Dummy,end of data of loaded nodes
c 10	0    0.0    0.0 		   Dummy,end of data of member loads

c    The letter c at the start of each data line  must, of course, be omitted.
c  To run the program , type SPACET and press Enter. The program will execute
c  the analysis and give the input and the results in a file named SPACET.OUT .

c			     Limits on Data
c    Vectors {B} and {NI} are used to store all the real and the integer arrays
c  used by the program. The maximum dimensions set are: MAXB=20000 and MAXNI=
c  2000 . This would normally be sufficient for a truss approximately 120
c  nodes,120 members and 8 load cases, depending upon the band width. The
c  maximum dimensions may be changed as required on line 2 of program(see
c  listing below).
c  The program must be compiled after the change using a fortran compiler
c  (e.g. of Microsoft).The dimension of the vectors used for a given problem
c  may be calculated by:

c  Dimension of {B}=NJ*(3+6*NLC+3*NB)+NM+3*NSJ*(NLC+2)+2*(NRAR+1)

c  Dimension of{NI}=2*NM+4*NSJ+2*(NRAR+1)

c------------------------------------------------------------------------------
c		     Notation and dimensions of arrays
c								     rows  cols.
c  {A}	  cross-section areas					     NM
c  [AR]   member end forces with nodal displacements restrained      2	   NRAR
c  [D]	  nodal displacements					    3*NJ   NLC
c  E	  modulus of elasticity
c  [ES]   element (member) stiffness in local coordinates	     2	    2
c  [EST]  element stiffness matrix in directions parallel to	     6	    6
c	  global axes
c  [F]	  nodal restraining forces				    3*NJ   NLC
c  [ISUP] joint restraint indicators: 1 and 0 ,indicating free	    NSJ    3
c	  and prescribed displacement, respectively
c  {JE}   end nodes of individual members			     NM
c  {JS}   starting node of individual members			     NM
c  [LR]   list of load case and member number for which fixed-end    2	   NRAR
c	  forces are given as data
c  NB     band width ; see Sec. 23-3 of Ghali and Neville.
c  NJ	  number of joints (nodes)
c  NLC	  number of load cases
c  NM	  number of members
c  NRAR   number of rows in [AR] ; this is equal the number of mem-
c	  bers for which fixed-end forces are given as data
c  {NS}   list of node numbers where supports are provided or	     NSJ
c	  displacements are prescribed
c  NSJ	  number of supported joints (nodes with prescribed
c	  displacements)
c  [PD]   prescribed displacement values			     NSJ   3
c  [R]	  forces at the supprted nodes including the reactions	    3*NSJ NLC
c  [S]	  stiffness matrix					    3*NJ   NB
c  {SK}   diagonal elements of [S] corresponding to  coordinates    3*NSJ
c	  at the supportd nodes
c  {X}	  x coordinates of nodes				     NJ
c  {Y}	  y coordinates of nodes				     NJ
c  {Z}	  z coordinates of nodes
c------------------------------------------------------------------------------

c	   Listing of program SPACET

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PARAMETER(MAXB=20000,MAXNI=2000)
      COMMON/REALV/B(MAXB)
      COMMON/NINTGR/NI(MAXNI)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      CHARACTER * 76 TITLE
      OPEN(5,FORM='FORMATTED',MODE='READ',FILE='SPACET.IN')
      OPEN(6,FORM='FORMATTED',MODE='WRITE',FILE='SPACET.OUT')
      DO 10 I=1,MAXB
10    B(I)=0.0
      DO 20 I=1,MAXNI
20    NI(I)=0
      READ(5,100)TITLE
100   FORMAT(A76)
      WRITE(6,200)TITLE
200   FORMAT(3X,A76)
      READ(5,*)NJ,NM,NSJ,NLC
      WRITE(6,300)NJ
300   FORMAT(3X,'Number of joints=',I5)
      WRITE(6,400)NM
400   FORMAT(3X,'Number of members=',I5)
      WRITE(6,500)NSJ
500   FORMAT(3X,'Number of joints with prescribed displacement(s)=',I5)
      WRITE(6,600)NLC
600   FORMAT(3X,'Number of load cases=',I5)
      NJ3=3*NJ
      NS3=3*NSJ
c     Elements of {B} at which real arrays start are:
c     {A}:N1  [D]:N2  [F]:N3  [PD]:N4  [R]:N5  {SK}:N6
c     {X}:N7  {Y}:N8  {Z}:N9  [S]:N10 [AR]:N11
      N1=1
      N2=N1+NM
      N3=N2+NJ3*NLC
      N4=N3+NJ3*NLC
      N5=N4+NS3
      N6=N5+NS3*NLC
      N7=N6+NS3
      N8=N7+NJ
      N9=N8+NJ
      N10=N9+NJ
c     Elements of {N} at which integer arrays start are:
c     {JS}:K1  {JE}:K2	{NS}:K3  [ISUP]:K4  [LR]:K5
      K1=1
      K2=K1+NM
      K3=K2+NM
      K4=K3+NSJ
      K5=K4+NS3
c     CALL DATAST(X,Y,Z,JS,JE,A,NS,ISUP,PD,NB)
      CALL DATAST(B(N7),B(N8),B(N9),NI(K1),NI(K2),B(N1),NI(K3),
     *NI(K4),B(N4),NB)
c     Identification of elements of {B} at which real arrays start
c     (continued)
      N11=N10+NJ3*NB
c     Determine maximum number of rows of [AR] that can be stored in
c     the remaining space in {B}. Also determine the maximum number of
c     rows of [R] which can be stored in the remaining space in {NI}.
      NAR=(MAXB-N11)/2
      NLR=(MAXNI-K5)/2
c     CALL STIFST(JS,JE,A,X,Y,Z,S,NJ3,NB)
      CALL STIFST(NI(K1),NI(K2),B(N1),B(N7),B(N8),B(N9),B(N10),
     *NJ3,NB)
c     CALL LOADST(JS,JE,X,Y,Z,F,LR,AR,NJ3,NLR,NAR,NLC)
      CALL LOADST(NI(K1),NI(K2),B(N7),B(N8),B(N9),B(N3),NI(K5),B(N11),
     *NJ3,NLR,NAR,NLC)
c     CALL SUPTST(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
      CALL SUPTST(B(N10),B(N3),NI(K3),NI(K4),B(N4),B(N5),B(N6),NJ3,NS3,
     *NB,NLC)
c     CALL EQSOLN(S,F,D,NJ3,NB,NLC)
      CALL EQSOLN(B(N10),B(N3),B(N2),NJ3,NB,NLC)
c     CALL RSLTST(D,NS,ISUP,PD,R,SK,X,Y,Z,JS,JE,A,LR,AR,NLR,NAR,
c    *NJ3,NS3,NLC)
      CALL RSLTST(B(N2),NI(K3),NI(K4),B(N4),B(N5),B(N6),B(N7),B(N8),
     *B(N9),NI(K1),NI(K2),B(N1),NI(K5),B(N11),NLR,NAR,NJ3,NS3,NLC)
      CLOSE(5)
      CLOSE(6)
      STOP
      END

      SUBROUTINE DATAST(X,Y,Z,JS,JE,A,NS,ISUP,PD,NB)
c     Reading data for plane frame (excluding loads)and calculation of
c     band width.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION X(NJ),Y(NJ),Z(NJ),JS(NM),JE(NM),A(NM)
      DIMENSION NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      READ(5,*)E
      WRITE(6,200)E
200   FORMAT(3X,'Elasticity modulus=',E13.5)
      WRITE(6,300)
300   FORMAT(/,3X,'Nodal coordinates')
      WRITE(6,400)
400   FORMAT(5X,'Node',6x,'x',9X,'y',9X,'z')
      DO 10 I=1,NJ
      READ(5,*)J,X(J),Y(J),Z(J)
10    WRITE(6,500)J,X(J),Y(J),Z(J)
500   FORMAT(3X,I5,3F10.3)
      WRITE(6,600)
600   FORMAT(/,3X,'Element information')
      WRITE(6,700)
700   FORMAT(3X,'Element',2X,'1st node',3X,'2nd node',6x,'a')
      DO 20 I=1,NM
      READ(5,*)J,JS(J),JE(J),A(J)
c     Member no.,starting node,end node,cross-section area,2nd moment
c     of area and area reduced.
20    WRITE(6,800)J,JS(J),JE(J),A(J)
800   FORMAT(3X,I5,4X,I5,5X,I5,3X,E13.5,1X)
      WRITE(6,900)
900   FORMAT(/,3X,'Support conditions')
      WRITE(6,1000)
1000  FORMAT(5X,'Node',5X,'Restraint indicators',7X,'Prescribed displace
     *ments')
c     Restraint indicators are 1 or o,meaning free or prescribed disp-
c     lacement,respectively.
      WRITE(6,1100)
1100  FORMAT(16x,'u',6x,'v',6X,'w',9x,'u',14x,'v',14x,'w')
1200  FORMAT(3X,I5,4X,I5,2X,I5,2X,I5,3(2X,E13.5))
      DO 30 J=1,NSJ
      READ(5,*)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),PD(J,
     *3)
30    WRITE(6,1200)NS(J),ISUP(J,1),ISUP(J,2),ISUP(J,3),PD(J,1),PD(J,2),
     *PD(J,3)
c     Determine band width by Eq. 23-1 of Ghali and Neville.
      NBI=0
      DO 50 I=1,NM
      N=JS(I)-JE(I)
      IF(N.GE.0) GO TO 40
      N=-N
40    CONTINUE
      IF(N.GT.NBI)NBI=N
50    CONTINUE
      NB=3*(NBI+1)
      RETURN
      END

      SUBROUTINE STIFST(JS,JE,A,X,Y,Z,S,NJ3,NB)
c     Stiffness matrix for space truss.See Sec. 23-9,Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION JS(NM),JE(NM),A(NM),X(NJ),Y(NJ),Z(NJ)
      DIMENSION S(NJ3,NB)
      DIMENSION EST(6,6),IM(6)
      N3=3*NJ
      DO 10 I=1,N3
      DO 10 J=1,NB
      S(I,J)=0.0
10    CONTINUE
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/ALJ
      CY=ALY/ALJ
      CZ=ALZ/ALJ
      AJ=A(JM)
c     Element stiffness, [EST] in global directions,Eq. 23-24 of Ghali
c     and Neville.
      DO 20 I=1,6
      DO 20 J=1,6
20    EST(I,J)=0.0
      Q=E*A(JM)/ALJ
      EST(1,1)=Q*CX*CX
      EST(2,1)=Q*CY*CX
      EST(3,1)=Q*CZ*CX
      EST(4,1)=-EST(1,1)
      EST(5,1)=-EST(2,1)
      EST(6,1)=-EST(3,1)
      EST(2,2)=Q*CY*CY
      EST(3,2)=Q*CZ*CY
      EST(4,2)=-EST(2,1)
      EST(5,2)=-EST(2,2)
      EST(6,2)=-EST(3,2)
      EST(3,3)=Q*CZ*CZ
      EST(4,3)=-EST(3,1)
      EST(5,3)=-EST(3,2)
      EST(6,3)=-EST(3,3)
      EST(4,4)=EST(1,1)
      EST(5,4)=EST(2,1)
      EST(6,4)=EST(3,1)
      EST(5,5)=EST(2,2)
      EST(6,5)=EST(3,2)
      EST(6,6)=EST(3,3)
      DO 30 I=1,6
      DO 30 J=I+1,6
30    EST(I,J)=EST(J,I)
c     Identify the six coordinates at the start and end of member.
      IM(1)=3*JS(JM)-2
      IM(2)=IM(1)+1
      IM(3)=IM(1)+2
      IM(4)=3*JE(JM)-2
      IM(5)=IM(4)+1
      IM(6)=IM(4)+2
      DO 40 I=1,6
      DO 40 J=1,6
      K=IM(J)-IM(I)
      IF(K.LT.0)GO TO 35
      K=K+1
      S(IM(I),K)=S(IM(I),K)+EST(I,J)
35    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END


      SUBROUTINE LOADST(JS,JE,X,Y,Z,F,LR,AR,NJ3,NLR,NAR,NLC)
c     Generation of vectors of restraining forces,[F]
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION JS(NM),JE(NM),X(NJ),Y(NJ),Z(NJ),F(NJ3,NLC),LR(2,NLR)
      DIMENSION AR(2,NAR)
c     Forces applied at the nodes.Note that the last line of this set
c     of input lines must start with an integer > number of load cases;
c     the line must be completed by another integer and 3 real numbers.
      WRITE(6,100)
100   FORMAT(/,3X,'Forces applied at the nodes')
      WRITE(6,200)
200   FORMAT(3X,'Load case ',3X,'Node',6X,'Fx',12X,'Fy',12X,'Fz')
20    CONTINUE
      READ(5,*)LCASE,NODE,FX,FY,FZ
c     IF(LCASE.GT.NLC)GO TO 30
      WRITE(6,300)LCASE,NODE,FX,FY,FZ
300   FORMAT(3X,I5,6X,I5,3(1X,E13.5))
      IF(LCASE.GT.NLC)GO TO 30
      N1=3*NODE-2
      N2=N1+1
      N3=N2+1
      F(N1,LCASE)=F(N1,LCASE)-FX
      F(N2,LCASE)=F(N2,LCASE)-FY
      F(N3,LCASE)=F(N3,LCASE)-FZ
      GO TO 20
30    CONTINUE
c     Fixed-end forces(given in data)due to temperature,shrinkage lack
c     of fit ,etc..Note that the last line in this set of data lines
c     must start with an integer greater than number of load cases;the
c     line must be completed by another integer and two real numbers.
c     The positive directions of the end forces are shown in Fig.23-2 of
c     Ghali and Neville.
      WRITE(6,400)
400   FORMAT(/,3X,'Member end forces with nodal displacement restrained
     *')
      WRITE(6,500)
500   FORMAT(2X,'Ld. case ',1x,'Member',6X,'Ar1',8X,'Ar2')
      NRAR=0
c     NRAR is number of rows in [AR] of fixed-end forces.
40    CONTINUE
      NRAR=NRAR+1
      N=NRAR
      READ(5,*)LR(1,N),LR(2,N),AR(1,N),AR(2,N)
      WRITE(6,600)LR(1,N),LR(2,N),AR(1,N),AR(2,N)
600   FORMAT(3X,I5,2X,I5,6X,E10.4,1x,E10.4)
      IF(LR(1,N).GT.NLC)GO TO 50
      LCASE=LR(1,N)
      MEMBER=LR(2,N)
c     Transform the fixed-end forces into global directions and add to
c     the existing vectors of restraining forces.To do this,identify the
c     node and coordinate numbers at the member ends and calculate the
c     cosine and sine the angle that the member makes with the global
c     x axis.
      N1=3*JS(MEMBER)-2
      N2=N1+1
      N3=N1+2
      ALX=X(JE(MEMBER))-X(JS(MEMBER))
      ALY=Y(JE(MEMBER))-Y(JS(MEMBER))
      ALZ=Z(JE(MEMBER))-Z(JS(MEMBER))
      AL=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/AL
      CY=ALY/AL
      CZ=ALZ/AL
      F(N1,LCASE)=F(N1,LCASE)+AR(1,N)*CX
      F(N2,LCASE)=F(N2,LCASE)+AR(1,N)*CY
      F(N3,LCASE)=F(N3,LCASE)+AR(1,N)*CZ
      N1=3*JE(MEMBER)-2
      N2=N1+1
      N3=N1+2
      F(N1,LCASE)=F(N1,LCASE)+AR(2,N)*CX
      F(N2,LCASE)=F(N2,LCASE)+AR(2,N)*CY
      F(N3,LCASE)=F(N3,LCASE)+AR(2,N)*CZ
      GO TO 40
50    CONTINUE
      NRAR=NRAR-1
      RETURN
      END

      SUBROUTINE SUPTST(S,F,NS,ISUP,PD,R,SK,NJ3,NS3,NB,NLC)
c     Adjust [S] and [F] in accordance with the support conditions.See
c     Eq. 23-39 of Ghali and Neville. Note that before the adjustment,
c     [S] generated by the subroutine STIFPF is the stiffness matrix of
c     a free unsupported structure.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION S(NJ3,NB),F(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC),SK(NS3)
      DO 10 I=1,NS3
      SK(I)=0.0
      DO 10 J=1,NLC
10    R(I,J)=0.0
c     Use {SK} and [R] to store appropriate elements on the diagonal of
c     [S] and of [F] for use in calculation of the reactions by Eqs. 23-
c     38 and 23-40 of Ghali and Neville.
c     For some trusses,elements on the diagonal of the stiffness matrix
c     can be zero. For this reason we replace Skk by 10e6*(Skk+1) and
c     and the correponding prescribed displacement by (Skk+1).In other
c     words,the parameter Sk in Eq.23-39 of Ghali and Neville is repl-
c     aced by (Sk+1) and Eq. 23-40 by:

c     Fk=10e6*(Skk+1)(Ck-Dk)+Skk*Dk

      DO 50 I=1,NSJ
      M1=3*I-3
      N1=3*NS(I)-3
      DO 40 K=1,3
      IF(ISUP(I,K).EQ.1)GO TO 30
      M=M1+K
      N=N1+K
      SK(M)=S(N,1)
      S(N,1)=(S(N,1)+1)*1.0E6
      DO 20 J=1,NLC
      R(M,J)=F(N,J)
      F(N,J)=-(SK(M)+1)*1.0E6*PD(I,K)
20    CONTINUE
30    CONTINUE
40    CONTINUE
50    CONTINUE
      RETURN
      END

      SUBROUTINE RSLTST(D,NS,ISUP,PD,R,SK,X,Y,Z,JS,JE,A,LR,AR,NLR,
     *NAR,NJ3,NS3,NLC)
c     Printing of nodal displacement reactions and member end forces.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON/STDAT1/NJ,NM,NSJ,E,NRAR
      DIMENSION D(NJ3,NLC),NS(NSJ),ISUP(NSJ,3),PD(NSJ,3)
      DIMENSION R(NS3,NLC)
      DIMENSION SK(NS3),X(NJ),Y(NJ),Z(NJ),JS(NM),JE(NM),A(NM)
      DIMENSION LR(2,NLR),AR(2,NAR)
      WRITE(6,1000)
1000  FORMAT(3X,'------------------------------------------------------
     *-------------------------------------------------')
      WRITE(6,1100)
1100  FORMAT(3X,'Analysis results')
      DO 60 K=1,NLC
      WRITE(6,100)K
100   FORMAT(/,3X,'Load case no.',I5)
      WRITE(6,200)
200   FORMAT(/,3X,'Nodal displacements')
      WRITE(6,300)
300   FORMAT(5X,'Node',9X,'u',15X,'v',15x,'w')
      DO 10 I=1,NJ
      N1=3*I-2
      N2=N1+1
      N3=N1+2
      WRITE(6,400)I,D(N1,K),D(N2,K),D(N3,K)
400   FORMAT(3X,I5,3(3X,E13.5))
10    CONTINUE
      WRITE(6,500)
500   FORMAT(/,3X,'Forces at the supported nodes')
      WRITE(6,600)
600   FORMAT(5X,'NODE',8X,'Fx',14X,'Fy',14X,'Fz')
      DO 30 L=1,NSJ
      M1=3*L-3
      N1=3*NS(L)-3
      DO 20 J=1,3
      IF(ISUP(L,J).EQ.1)GO TO 20
      M=M1+J
      N=N1+J
      R(M,K)=R(M,K)+(SK(M)+1)*1.0E6*(PD(L,J)-D(N,K))+SK(M)*D(N,K)
20    CONTINUE
      M1=M1+1
      M2=M1+1
      M3=M1+2
      WRITE(6,400)NS(L),R(M1,K),R(M2,K),R(M3,K)
30    CONTINUE
      WRITE(6,700)
700   FORMAT(/,3X,'Member end forces')
      WRITE(6,800)
800   FORMAT(3X,'Member',7X,'F1',10X,'F2')
      DO 50 JM=1,NM
      ALX=X(JE(JM))-X(JS(JM))
      ALY=Y(JE(JM))-Y(JS(JM))
      ALZ=Z(JE(JM))-Z(JS(JM))
      ALJ=DSQRT(ALX*ALX+ALY*ALY+ALZ*ALZ)
      CX=ALX/ALJ
      CY=ALY/ALJ
      CZ=ALZ/ALJ
      AJ=A(JM)
      Q=E*A(JM)/ALJ
      N1=3*JS(JM)-2
      N2=N1+1
      N3=N1+2
      N4=3*JE(JM)-2
      N5=N4+1
      N6=N4+2
c     Transform displacements at member ends into local coordinates.
      DT1=D(N1,K)*CX+D(N2,K)*CY+D(N3,K)*CZ
      DT2=D(N4,K)*CX+D(N5,K)*CY+D(N6,K)*CZ
c     For each member,calculate the product of its stiffness matrix by
c     its displacement vector.
      A1=Q*DT1-Q*DT2
      A2=-Q*DT1+Q*DT2
c     Add the fixed-end forces included in the input data.
      IF(NRAR.EQ.0)GO TO 45
      DO 40 I=1,NRAR
      IF(LR(1,I).NE.K) GO TO 35
      IF(LR(2,I).NE.JM) GO TO 35
      A1=A1+AR(1,I)
      A2=A2+AR(2,I)
35    CONTINUE
40    CONTINUE
45    CONTINUE
      WRITE(6,900)JM,A1,A2
900   FORMAT(3X,I5,2X,2(1X,E11.5))
50    CONTINUE
      WRITE(6,1200)
1200  FORMAT(3X,'-------------------------------------------------------
     *------------------------------------------------')
60    CONTINUE
      RETURN
      END
c__________________________________________________________________________

c		  Solution of Banded Symmetrical Equations
c		      For Use in Structural Analysis
c----------------------------------------------------------------------
c		     Notation and dimensions of arrays
c							     rows  cols.
c    NB     band width ; see Sec. 23-3 of Ghali and Nevile.
c    NLC    number of load cases
c    [D]    nodal displacements 			     NDF    NLC
c    [F]    nodal restraining forces			     NDF    NLC
c    [S]    stiffness matrix				     NDF    NB
c------------------------------------------------------------------------------
c			  Program Listing
c
      SUBROUTINE EQSOLN(S,F,D,NDF,NB,NLC)
c     Solution of equilibrium equations [S][D]=-[F], using Cholesky's
c     method; see Sec. 23-11 of Ghali and Neville.
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION S[HUGE](NDF,NB),F(NDF,NLC),D(NDF,NLC)
      IF(S(1,1).LE. 0.0)GO TO 98
      DO 50 J=2,NDF
      JJ=J-1
      JJJ=J-NB+1
      IF(JJJ.LT.1)JJJ=1
      IF(JJ.EQ.1)GO TO 30
      DO 20 I=2,JJ
      II=I-1
      IF(II.LT.JJJ)GO TO 20
      SUM=S(I,J-I+1)
      DO 10 K=JJJ,II
10    SUM=SUM-S(K,I-K+1)*S(K,J-K+1)
      S(I,J-I+1)=SUM
20    CONTINUE
30    SUM=S(J,1)
      DO 40 K=JJJ,JJ
      T=S(K,J-K+1)/S(K,1)
      SUM=SUM-T*S(K,J-K+1)
40    S(K,J-K+1)=T
c     IF(SUM.LE.0.0)GO TO  98
      if(sum.lt.0.0)write(6,101)j
      if(sum.eq.0.0)go to 98
50    S(J,1)=SUM
c     At this stage,the first column of the band matrix [S] is replaced
c     by e11,e22,... and the remaining elements of the same matrix are
c     replaced by  elements uij using Eq. 23-49 to 23-53 of Ghali and
c     Neville. Now by forward substitution (Eq. 23-48), determine [g]
c     and store in [D].
      DO 95 L=1,NLC
      DO 70 I=1,NDF
      J=I-NB+1
      IF(I.LE.NB)J=1
      SUM=-F(I,L)
      KK=I-1
      IF(J.GT.KK)GO TO 65
      DO 60 K=J,KK
60    SUM=SUM-S(K,I-K+1)*D(K,L)
65    D(I,L)=SUM
70    CONTINUE
      DO 80 I=1,NDF
80    D(I,L)=D(I,L)/S(I,1)
c     By Eq. 23-46 of Ghali and Neville and backward substitution(Eq.23-
c     47),determine the required displacement values and store in [D].
      DO 95 II=1,NDF
      I=NDF-II+1
      J=I+NB-1
      IF(J.GT.NDF) J=NDF
      SUM=D(I,L)
      KK=I+1
      IF(KK.GT.J) GO TO 90
      DO 85 K=KK,J
85    SUM=SUM-S(I,K-I+1)*D(K,L)
90    D(I,L)=SUM
95    CONTINUE
      GO TO 99
98    CONTINUE
      WRITE(6,102)J
102   FORMAT(/,3X,
     +'Stiffness matrix is not positive definite. Zero value encountered
     +',/,3X,
     +'on the diagonal in solution of equations on row:',I4)
101   format(/,3x,
     +'Stiffness matrix is not positive definite. Negative value encount
     +ered',/,3X,
     +'on the diagonal in solution of equations on row:',I4)
      STOP
99    CONTINUE
      RETURN
      END


